/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;

import graphe.Graphe;

import java.util.ArrayList;

import cout.CoutMult;
/**
 * Classe correspondant au calcul de la solution en e
 * 
 * @author Giraudet
 *
 */
public class CalculCoutMultiplicatif extends MethodeResolution{

	private ArrayList<CoutMult> lcm;
	private boolean spec=false;
	
	public CalculCoutMultiplicatif(Graphe g)
	{
		super(g);
	}
	
	public CalculCoutMultiplicatif(Graphe g,ArrayList<CoutMult> _lcm) {
		super(g);
		spec=true;
		lcm=_lcm;
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	
	/**
	 * pre-condition taille(lvalcalcule)==taille(lideal)
	 */
	public double distance(ArrayList<Double> lvalcalcule,int j)
	{

		double res=0.0;
		for(int i=0;i<lcm.size();i++)
			res+=Math.pow(lvalcalcule.get(i+j)-lcm.get(i).getIdeal().moyenne(),2);
		return res;
	}
	
	
	/**
	 * pre-condition les matrices de cout et la matrice m ont les memes dimensions
	 * fonction qui multiplie la matrice par les differentes matrices de couts 
	 * @param lcout
	 * @param m
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public ArrayList<Matrice<Double>> multcout(Matrice<Double> m)
	{
		ArrayList<Matrice<Double>> lm = new ArrayList<Matrice<Double>>();
		//on parcourt la liste de couts
		for(int i=0;i<lcm.size();i++)
		{
			Matrice<Double> cout = lcm.get(i).getMatrice();
			Matrice<Double> m1 = appliqueCoutMatrice(m, cout);
			lm.add(m1);
		}
		return lm;
	}

	/**
	 * calcul de la matrice resultante de la matrice de probas avec une matrice de cout donnee
	 */
	public Matrice<Double> appliqueCoutMatrice(Matrice<Double> md, Matrice<Double> mc)
	{
		Matrice<Double> res = new Matrice<Double>(md.getNbLigne(),md.getNbColonne());
		for(int j=0;j<res.getNbLigne();j++)
		{
			for(int l=0;l<res.getNbColonne();l++)
			{
				if(mc.getCase(j,l)!=1)
				{   
					double x = md.getCase(j,l);
					res.setCase(j,l,x*mc.getCase(j,l));
				}
				else
					res.setCase(j,l,md.getCase(j,l));
			}
		}
		return res;
	}
	
	/**
	 * fonction qui retourne le vecteur resulat en calculant les valeurs propres de chaque matrice
	 * contenu dans la liste lm
	 */
	public void calculVecteurK (ArrayList<Matrice<Double>> lm,ArrayList<Double> vecteurK)
	{
		//cette fonction est utilise en premier par cette méthode si elle est employe
		for (int i=0;i<lm.size();i++)
			vecteurK.add(lm.get(i).CalculValeurPropre());
	}
	
	@Override
	public boolean estSolution(Matrice<Double> m, ArrayList<Double> vecteurKCalcule) {
		if(m==null)
			return false;
		else
		{
			ArrayList<Matrice<Double>> lcal = multcout(m);
			int j=0;
			if(vecteurKCalcule==null)
				vecteurKCalcule = new ArrayList<Double>();
			else
				j=vecteurKCalcule.size();
			calculVecteurK(lcal,vecteurKCalcule);
			double x = distance(vecteurKCalcule,j);
			return (Math.abs(x)<EPSILON && Graphe.respectRelation(relations,m));
		}
	}

	@Override
	public double heuristique(Matrice<Double> m) {
		ArrayList<Matrice<Double>> linter = multcout(m);
		//on calcule le vecteur resultat correspondant
		ArrayList<Double> vecteurK = new ArrayList<Double>();
		calculVecteurK(linter,vecteurK);
		double dist = distance(vecteurK,0); 
		return dist + Graphe.evalueMatrice(relations,m);
	}

	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return "CalculCoutMultiplicatif";
	}
	
	@Override
	public void prepareRechercheLocale() {
		super.prepareRechercheLocale();
		if(lcm==null)
			lcm=_g.coutMultActif();
	}

	@Override
	public void finRechercheLocale()
	{
		super.finRechercheLocale();
		if(cpt==0 && !spec)
		{
			lcm=null;
		}
		
	}
}
