/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;

import java.util.ArrayList;
import java.util.concurrent.Semaphore;

import ihm.dessin.DessinGraphe;
import ihm.intervalle.Intervalle;
/**
 * Classe qui permet de calculer la coherence de borne ce qui permet d'avoir une approximation de 
 * l'ensemble solution
 * @author giraudet-docet
 *
 */
public class CoherenceBornes extends Thread
{
	private static final double PAS = 0.001;
	private Matrice<Intervalle> matriceIntervalles;
	private ArrayList<MethodeResolution> lmr;
	private static int num=0;
	private int numero;
	public Semaphore s;

	public CoherenceBornes(DessinGraphe g)
	{
		num++;
		numero=num;
		matriceIntervalles = g.graphe.construireMatriceIntervalle();
		lmr= new ArrayList<MethodeResolution>();
		lmr.add(new UsingMarks(g.graphe));
		for(int i=1;i<g.methodes.size();i++)
			lmr.add(g.methodes.get(i));
		s = new Semaphore(1);
		try {
			s.acquire();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public int getnum()
	{
		return numero;
	}
	
	public Matrice<Intervalle> getMat()
	{
		return matriceIntervalles;
	}
	
	/*
	 * On parcourt la matrice d'intervalle on bloque l'intervalle à la borne min
	 * on essaye de voir s'il y a une solution en effectuant une recherche locale
	 * si il y en a une alors on garde la borne min, sinon on passe au pas suivant
	 * et respectivement pour la borne max
	 * (non-Javadoc)
	 * @see java.lang.Thread#run()
	 */
	public void run()
	{
		
		long t = System.currentTimeMillis();
		for(int i=0; i<matriceIntervalles.getNbLigne(); i++)
		{	for(int j=0; j<matriceIntervalles.getNbColonne(); j++)
			{
				if(isInterrupted())
					break;
				Intervalle inter = new Intervalle(trouveLimiteMin(i, j)*100,trouveLimiteMax(i, j)*100);
				matriceIntervalles.setCase(i, j, inter);
			}
		if(isInterrupted())
			break;
		}
		System.out.println("time for consistency n°"+numero+" : " + (System.currentTimeMillis()-t));
		s.release();
	}

	public double trouveLimiteMin(int l, int c)
	{
		Matrice<Double> m;
		Matrice<Intervalle> mi = new Matrice<Intervalle>(matriceIntervalles);
		mi.setCase(l, c, new Intervalle(matriceIntervalles.getCase(l, c).getMin(),matriceIntervalles.getCase(l, c).getMin()));
		double res = matriceIntervalles.getCase(l, c).getMinProba();
		mi.setCase(l, c, new Intervalle(res*100,res*100));
		m = calculSolution(mi);
		while(m==null)
		{
			if(isInterrupted())
				break;
			res += PAS;
			if((res>matriceIntervalles.getCase(l, c).getMaxProba())||!(sommeligne(l,c,res)))
				return matriceIntervalles.getCase(l, c).getMinProba();
			mi.setCase(l, c, new Intervalle(res*100,res*100));

			m = calculSolution(mi);
		}
		return res;
	}

	public double trouveLimiteMax(int l, int c)
	{
		Matrice<Double> m;
		Matrice<Intervalle> mi = new Matrice<Intervalle>(matriceIntervalles);
		mi.setCase(l, c, new Intervalle(matriceIntervalles.getCase(l, c).getMax(),matriceIntervalles.getCase(l, c).getMax()));
		double res = matriceIntervalles.getCase(l, c).getMaxProba();
		mi.setCase(l, c, new Intervalle(res*100,res*100));
		m = calculSolution(mi);
		while(m==null)
		{
			if(isInterrupted())
				break;
			res -= PAS;
			if((res<matriceIntervalles.getCase(l, c).getMinProba())||!(sommeligne(l,c,res)))
				return matriceIntervalles.getCase(l, c).getMaxProba();
			mi.setCase(l, c, new Intervalle(res*100,res*100));
			m = calculSolution(mi);
		}
		return res;
	}




	public Matrice<Double> calculSolution(Matrice<Intervalle> mi)
	{
		if(!isInterrupted())
		{
		RechercheLocale rl = new RechercheLocale(mi,lmr, DessinGraphe.EPSILON,num);
		rl.start();
		while(!rl.s.tryAcquire())
		{
			if(isInterrupted())
			{
				rl.interrupt();
				break;
				}
		}

		return rl.resultat;
		}
		return null;
	}

	public boolean sommeligne(int l,int c,double res)
	{
		double sommeMin=res;
		double sommeMax=res;
		for(int j=0;j<matriceIntervalles.getNbColonne();j++)
			if(j!=c)
			{
				Intervalle i =matriceIntervalles.getCase(l,j);
				sommeMax+=i.getMaxProba();
				sommeMin+=i.getMinProba();
			}
		return ((sommeMin<=1)&&(sommeMax>=1));
	}
}
