/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;
import java.util.ArrayList;

public class Gauss {

	private Matrice<Double> _m;
	private ArrayList<Double> _res;
	
	public Gauss(Matrice<Double> m,ArrayList<Double> res)
	{
		_m=new Matrice<Double>(m);
		_res=res;
	}
	
	public ArrayList<Double> ResoudreSysteme()
	{
		ArrayList<Double> vect = new ArrayList<Double>();
		for(int i=0;i<_m.getNbColonne()-1;i++)
			vect.add(0.0);
		vect.add(1.0);
		for(int i=0;i<_m.getNbLigne()-1;i++)
		{
			if(nbCoefNonNul(i)!=0)
			{
				double pivot = _m.getCase(i, i);
				normalize(i, pivot);
				recurrencePivot(i);
			}
		}
		for(int i=_m.getNbLigne()-2;i>=0;i--)
		{
			vect.set(i,resCoef(i));
		}
			
			return vect;
	}
	
	public int nbCoefNonNul(int i)
	{
		int nb=0;
		for(int j=i;j<_m.getNbColonne()-1;j++)
			if(_m.getCase(i, j)!=0.0)
				nb++;
		return nb;
	}
	
	public double resCoef(int i)
	{
		double x=_res.get(i);
		for(int j=0;j<i;j++)
			if(_m.getCase(i, j)!=0.0)
				x+=-_m.getCase(i, j);
		for(int j=i+1;j<_m.getNbColonne();j++)
			if(_m.getCase(i, j)!=0.0)
				x+=-_m.getCase(i, j);
		if(_m.getCase(i, i)!=0.0)
			x/=_m.getCase(i,i);
		return x;
	}
	
	public void normalize(int i,double pivot)
	{
		for(int j=0;j<_m.getNbColonne();j++)
		{
			_m.setCase(i, j, _m.getCase(i, j)/pivot);
		}
	}
	
	public void recurrencePivot(int i)
	{
		for(int j=i+1;j<_m.getNbLigne()-1;j++)
		{
			double pivot = _m.getCase(j, i);
			for(int c=0;c<_m.getNbColonne();c++)
				_m.setCase(j, c, _m.getCase(j, c)-pivot*_m.getCase(i, c));
		}
	}
	

}
