/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package calcul;
import ihm.intervalle.Intervalle;
import java.io.Serializable;
import java.util.*;



/**
 * 
 * Classe representant les matrices generiques
 * @author Docet Giraudet
 *
 * @param <T>
 */
public class Matrice <T> implements Serializable//,Comparable<Matrice<T>>
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private ArrayList<ArrayList<T>> _tab;
	
	//Constructeurs
	/**
	 * constructeur vide
	 */
	public Matrice()
	{
	}
	/**
	 * construit une matrice vide de l lignes et c colonnes
	 * @param l
	 * @param c
	 */
	@SuppressWarnings("unchecked")
	public Matrice(int l, int c)
	{
		_tab = new ArrayList<ArrayList<T>>(l);
	
		for(int i=0; i<l; i++)
		{
			ArrayList<T> aux = new ArrayList<T>(c);
			for(int j=0; j<c; j++)
			{
				aux.add(j,null);			
			}
			_tab.add(i,aux);
		}
	}
	/**
	 * constructeur qui recopie en mémoire une matrice m
	 * @param m
	 */
	@SuppressWarnings("unchecked")
	public Matrice(Matrice<T> m)
	{
		int l = m.getNbLigne();
		int c = m.getNbColonne();
		_tab = new ArrayList<ArrayList<T>>(l);

		for(int i=0; i<l; i++)
		{
			ArrayList aux = new ArrayList(c);
			for(int j=0; j<c; j++)
			{
				aux.add(j,m.getCase(i,j));		
			}
			_tab.add(i,aux);
		}
	}
	
	//Modifieurs
	/**
	 * permet de modifier un élément de la matrice de la case l,c
	 */
	@SuppressWarnings("unchecked")
	public Matrice setCase(int l, int c, T elt)
	{
		ArrayList aux = (ArrayList)_tab.get(l);
		aux.set(c,elt);
		_tab.set(l,aux);
		return this;
	}
	
	
	//Accesseurs
	public T getCase(int l, int c)
	{
		ArrayList<T> aux = _tab.get(l);
		return aux.get(c);
	}
	public int getNbLigne()
	{
		return _tab.size();
	}
	public int getNbColonne()
	{
		ArrayList<T> aux = _tab.get(0);
		return aux.size();
	}
		
	//Supprimer une ligne
	@SuppressWarnings("unchecked")
	public Matrice supprLigne(int l)
	{
		this._tab.remove(l);
		return this;
	}
	
	//Supprimer une colonne
	@SuppressWarnings("unchecked")
	public Matrice supprColonne(int c)
	{
		for(int i=0; i<this.getNbLigne(); i++)
		{
			ArrayList<T> aux = this._tab.get(i);
			aux.remove(c);
			this._tab.set(i,aux);			
		}
		return this;
	}
	//Afficheur
	public String toString()
	{
		String s = "";
		for(int i=0; i<this.getNbLigne(); i++)
		{
			ArrayList<T> aux = _tab.get(i);
			for(int j=0; j<this.getNbColonne(); j++)
			{
				s = s + aux.get(j) + "  "; 
			}
			s+="\n";
		}
		return s;
	}
	/**
	 * Fonction valable que pour des matrices carrees
	 * Calcul le polynome de la matrice
	 * @param m
	 * @return
	 */
	public Polynome CalculPolynome()
	{
		Polynome aux = new Polynome();
		Polynome truc;
		Matrice<Polynome> m = new Matrice<Polynome>(this.getNbLigne(),this.getNbColonne());
		for(int i=0;i<m.getNbColonne();i++)
		{
			for(int j=0;j<m.getNbLigne();j++)
			{
				truc = new Polynome();
				T t = this.getCase(i,j);
				Double val = new Double(t.toString());
				if(val!=null)
				{
					truc.setCoeff(val,0);
				}
				else 
					truc.setCoeff(0.0,0);
				if(i==j)
				{
					truc.setCoeff(-1,1);

					m.setCase(i,j,truc);
				}
				else
				m.setCase(i,j,truc);
			}
		}
		aux=Determinant(m);
		return aux;
	}
	/**
	 * calcul la valeur propre de la matrice
	 * @return
	 */
	public double CalculValeurPropre()
	{
		Polynome p = this.CalculPolynome();
		return p.RacineDominante();
	}
	/**
	 * calcule le determinant d'une matrice de polynome
	 * @param m
	 * @return
	 */
	static public Polynome Determinant(Matrice<Polynome> m)
	{
		Polynome d=new Polynome();
		d.setCoeff(0,0);
		if((m.getNbColonne()<=1)||(m.getNbLigne()<=1))
				{
			System.out.println("erreur de calcul");
					return null;
				}
		for (int i=0;i<m.getNbColonne();i++)
		{
			if((m.getNbColonne()==2)&&(m.getNbLigne()==2))
			{
				d=(m.getCase(0,0).mult(m.getCase(1,1))).sub(m.getCase(0,1).mult(m.getCase(1,0)));
				
				return d;
			}
			if(!m.getCase(0,i).PolynomeNul())
			{
				Matrice<Polynome> m1=new Matrice<Polynome>(m);
				m1.supprColonne(i);
				m1.supprLigne(0);
				if(i%2==0)
					d=d.add(m.getCase(0,i).mult(Determinant(m1)));
				else
					d=d.sub(m.getCase(0,i).mult(Determinant(m1)));
			}
		}
		return d;
	}
	
	/**
	 * calcul la somme de la ligne l d'une matrice de double m
	 * @param m
	 * @param ligne
	 * @return
	 */
	static public double sommeligne(Matrice<Double> m,int ligne)
	{
		double d=0.0;
		if (ligne<m.getNbLigne())
		{
			for (int i=0;i<m.getNbColonne();i++)
			{
				if(m.getCase(ligne,i)!=null)
				{
					d+=m.getCase(ligne,i);
				}
			}
		}
		return d;
	}
	
	/**
	 * calcul le determinant d'une matrice de reel
	 * @param m
	 * @return
	 */
	@SuppressWarnings("unchecked")
	static public double Determinant(Matrice<Double> m)
	{
		double d=0;
		if((m.getNbColonne()<=1)||(m.getNbLigne()<=1))
				{
			System.out.println("erreur de calcul");
					return d;
				}
		for (int i=0;i<m.getNbColonne();i++)
		{
			if((m.getNbColonne()==2)&&(m.getNbLigne()==2))
			{
				d=(m.getCase(0,0)*(m.getCase(1,1)))-(m.getCase(0,1)*(m.getCase(1,0)));
				
				return d;
			}
			if(!(m.getCase(0,i)==0))
			{
				Matrice<Double> m1=new Matrice<Double>(m);
				m1.supprColonne(i);
				m1.supprLigne(0);
				if(i%2==0)
					d=d+(m.getCase(0,i)*(Determinant(m1)));
				else
					d=d-(m.getCase(0,i)*(Determinant(m1)));
			}
		}
		return d;
	}
	
	/**
	 * permet  d'ajouter deux matrices deux reels
	 * @param m
	 * @param m1
	 * @return
	 */
	static public Matrice<Double> addMatriceReel (Matrice<Double> m, Matrice<Double> m1)
	{
		Matrice<Double> m2 = new Matrice<Double>(m.getNbColonne(),m.getNbLigne());
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne();j++)
			{
				m2.setCase(i,j,new Double(new Double((m1.getCase(i,j)+m.getCase(i,j))).floatValue()));
			}
		}
		return m2;
	}
	
	/**
	 * permet d'ajouter deux matrices d'intervalles
	 * @param m
	 * @param m1
	 * @return
	 */
	static public Matrice<Intervalle> addMatriceIntervalle (Matrice<Intervalle> m,Matrice<Intervalle> m1)
	{
		Matrice<Intervalle> m2 = new Matrice<Intervalle>(m.getNbColonne(),m.getNbLigne());
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne();j++)
			{
				if(m.getCase(i,j)==null)
				{
					if(m1.getCase(i,j)==null)
						m2.setCase(i,j,null);
					else
						m2.setCase(i,j,m1.getCase(i,j));
				}
				else
					if(m1.getCase(i,j)==null)
						m2.setCase(i,j,m.getCase(i,j));
					else
					{
						Intervalle inter = new Intervalle();
						Intervalle i1 = m.getCase(i,j);
						Intervalle i2 = m1.getCase(i,j);
						inter.setMax(i1.getMax()+i2.getMax());
						inter.setMin(i1.getMin()+i2.getMin());
						m2.setCase(i,j,inter);
					}
				
			}
		}
		return m2;
	}
	
	/**
	 * permet de multiplier deux matrices de reelss
	 * @param m
	 * @param m1
	 * @return
	 */
	static public Matrice<Double> multMatriceReel (Matrice<Double> m, Matrice<Double> m1)
	{
		Matrice<Double> m2 = new Matrice<Double>(m1.getNbColonne(),m.getNbLigne());
		Double x=0.0;
		for(int i=0;i<m.getNbLigne();i++)
		{			
			for(int l=0;l<m1.getNbColonne();l++)
			{
				for(int j=0;j<m.getNbColonne();j++)
					x+=m.getCase(i,j)*m1.getCase(j,l);
			m2.setCase(i,l,x);

			x=0.0;
			}
		}
		return m2;
	}
	
	/**
	 * permet de multiplier deux matrices d'intervalles
	 * @param m
	 * @param m1
	 * @return
	 */
	static public Matrice<Intervalle> multMatriceInter (Matrice<Intervalle> m, Matrice<Intervalle> m1)
	{
		Matrice<Intervalle> m2 = new Matrice<Intervalle>(m1.getNbColonne(),m.getNbLigne());
		Intervalle x = new Intervalle();
		x.setMax(0);
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int l=0;l<m1.getNbColonne();l++)
			{
				for(int j=0;j<m.getNbColonne();j++)
				{
					if((m.getCase(i,j)!=null)&&(m1.getCase(j,l)!=null))
					{
						Intervalle i1 = m.getCase(i,j);
						Intervalle i2 = m1.getCase(j,l);
						x.setMax(x.getMax()+(int)((new Double(i1.getMax())/100)*(new Double(i2.getMax())/100)*100));
						x.setMin(x.getMin()+(int)((new Double(i1.getMin())/100)*(new Double(i2.getMin())/100)*100));
					}
				}
				m2.setCase(i,l,x);
				x=new Intervalle();
				x.setMax(0);
			}
		}
		return m2;
	}
	
	/**
	 * pre cond les deux matrices ont la meme taille
	 * permet de comparer deux matrices de reels
	 * @param m
	 * @param m1
	 * @param epsilon
	 * @return
	 */
	static public boolean CompareMatriceDouble(Matrice<Double> m,Matrice<Double> m1,double epsilon)
	{
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne();j++)
			{
				if(Math.abs(m.getCase(i,j)-m1.getCase(i,j))>epsilon)
					return false;
			}
		}
		return true;
	}
	
	/**
	 * permet de mettre a zero une matrice d'entier
	 * @param m
	 */
	static public void MiseAZero(Matrice<Integer> m)
	{
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne();j++)
				m.setCase(i,j,0);
		}
	}
	
	/**
	 * permet de mettre a un une matrice de double
	 * @param m
	 */
	static public void MiseAUn(Matrice<Double> m)
	{
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne();j++)
				m.setCase(i,j,1d);
		}
	}
	
	/**
	 * fonction qui permet de renvoyer une chaine dans le format csv
	 */
	public String toCSV()
	{
		String res ="";
		for(int i=0;i<getNbLigne();i++)
		{
			for(int j=0;j<getNbColonne()-1;j++)
			{
				if(getCase(i, j) == null)
					res = res + " - ;";
				else
					res = res + getCase(i, j).toString() +"; ";
			}
			res = res + getCase(i, getNbColonne()-1) + "\n";
		}	
		return res;
	}
	
	/**
	 * fonction qui calcul l'application d'un cout c sur une matrice m 
	 * @param c
	 * @param m
	 * @return
	 */
	public static Double multcout(Matrice<Integer> c,Matrice<Double> m)
	{
		Matrice<Double> m1 = new Matrice<Double>(m.getNbColonne(),m.getNbLigne());
		for(int j=0;j<m1.getNbLigne();j++)
		{
			for(int l=0;l<m1.getNbColonne();l++)
			{
				if(c.getCase(j,l)!=0)
				{   //si le cout est different de 0 alors on multiplie la case de la matrice par 1+epsilon
					double x = m.getCase(j,l);
					m1.setCase(j,l,x*Math.pow(1+0.00001,c.getCase(j,l)));
				}
				else
					//sinon on ne fait rien
					m1.setCase(j,l,m.getCase(j,l));
			}
		}
		return ((m1.CalculValeurPropre()-1)/0.00001);
	}
	
	/**
	 * Calcul un vecteur propre de matrice de double
	 *
	 */
	public static ArrayList<Double> calculVecteurPropre(Matrice<Double> m)
	{
		Double lambda = new Double((float)m.CalculValeurPropre());
		EigenVector x = new EigenVector(m);
		Matrice<Double> v = x.getEigenVector();
		Matrice<Double> evm = Matrice.multMatriceReel(m, v);
		int k=0;
		for(int j=0;j<v.getNbColonne();j++)
		{
			boolean trouve=true;
			for(int i=0;i<v.getNbLigne();i++)
			{
				Double evmc = evm.getCase(i, j);
				Double vcl = v.getCase(i,j)*lambda;
				if(!((vcl<=(evmc+0.000001))&&(vcl>=(evmc-0.000001))))
				{
					trouve=false;
					break;
				}
			}
			if(trouve)
			{
				k=j;
				break;
			}
		}
		ArrayList<Double> ev = new ArrayList<Double>();
		for(int i=0;i<v.getNbLigne();i++)
		{
			ev.add(v.getCase(i, k));
		}
		return ev;
	}
	
	/**
	 * Calcul le vecteur propre par un pivot de gauss
	 * @param m
	 * @return
	 */
	public static ArrayList<Double> calculVecteurPropreGauss(Matrice<Double> m)
	{
		Double lambda=new Double((float)m.CalculValeurPropre());
		Matrice<Double> identite = new Matrice<Double>(m.getNbLigne(),m.getNbColonne());
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne();j++)
				if(i==j)
					identite.setCase(i,i,1.0);
				else
					identite.setCase(i, j, 0.0);
		}
		identite=multParCoef(identite, -lambda);
		identite=addMatriceReel(m, identite);
		ArrayList<Double> res = new ArrayList<Double>();
		for(int i=0;i<m.getNbColonne();i++)
			res.add(0.0);
		Gauss g = new Gauss(identite,res);
		
		res = g.ResoudreSysteme();
		double somme = 0;
		for(int i=0;i<res.size();i++)
			somme+=res.get(i);
		for(int i=0;i<res.size();i++)
			res.set(i, res.get(i)/somme);
		return res;
			
	}
	
/**
 * fonction qui calcul le phi utiliser dans la formule du calcul pour les requetes :
 * etre passe au moins une fois par un etat ou un chemin
 * etre passe une unique fois par un etat ou un chemin
 * n'etre pas passe par un etat ou un chemin
 * @param m
 * @param nbEtat
 * @return
 */
	public static Double calculPhi(Matrice<Double> m, int nbEtat)
	{
		Double res=0.0;
		
		ArrayList<Double> p = new ArrayList<Double>();
		for(int i=0; i<nbEtat; i++)
			p.add(new Double(1d/nbEtat));
		
		ArrayList<Double> phiFleche = calculVecteurPropre(m);
		
		for(int i=0; i<nbEtat; i++)
			res += p.get(i)*phiFleche.get(i);
		return res;
	}
	
	/**
	 * fonction qui calcule le dominantEigenSpace
	 * @param matrice
	 * @param eigenvalue
	 * @return
	 */
	public static ArrayList<Double> DominantEigenspace(Matrice<Double> matrice, double eigenvalue) {
		// Par le pivot de Gauss
		// On attend une matrice carree
		// Par rapport a une valeur propre donnee
		System.out.println("eigenvalue : "+eigenvalue);
		for (int i=0; i<matrice.getNbColonne(); i++) {
			matrice.setCase(i,i,matrice.getCase(i,i)-eigenvalue);
		}
		
		for (int i=0; i<matrice.getNbColonne(); i++) {
			// Normalisation
			double pivot=matrice.getCase(i,i);
			for (int j=0; j<matrice.getNbColonne(); j++) 
				matrice.setCase(i,j,matrice.getCase(i,j)/pivot);
			
			// division
			for (int iprim=0; iprim<matrice.getNbColonne(); iprim++) 
			{
				if (i!=iprim) 
				{
					double mult=matrice.getCase(iprim,i);
					for (int j=i; j<matrice.getNbColonne(); j++) 
					{
						matrice.setCase(iprim,j,matrice.getCase(iprim,j)-mult*matrice.getCase(i,j));
					}
				}
			}
			
		}
		
		System.out.println(matrice);
		ArrayList<Double> res=new ArrayList<Double>(matrice.getNbColonne());
		for (int i=0; i<matrice.getNbColonne(); i++) 
		{
			res.add(-matrice.getCase(i,matrice.getNbColonne()-1));
		}
		res.add(new Double(1));
		
		return res;
	}
	
	/**
	 * fonction qui permet de normaliser une matrice
	 * @param m
	 * @return
	 */
	@SuppressWarnings("unchecked")
	static public Matrice<Double> normalize(Matrice<Double> m) {
		Matrice aux = new Matrice(m.getNbLigne(),m.getNbColonne());
		double somme=0;
		for (int i=0; i<m.getNbLigne(); i++) {
			for (int j=0; j<m.getNbColonne(); j++) {
				somme+=m.getCase(i,j);
			}
		}
		for (int i=0; i<m.getNbLigne(); i++) {
			for (int j=0; j<m.getNbColonne(); j++) {
				aux.setCase(i,j,m.getCase(i,j)/somme);
			}
		}
		return aux;
	
	}
	
	/**
	 * fonction qui multiplie une matrice par un coef
	 * @param m
	 * @param coef
	 * @return
	 */
	static public Matrice<Double> multParCoef(Matrice<Double> m, double coef)
	{
		Matrice<Double> res = new Matrice<Double>(m.getNbLigne(),m.getNbColonne());
		for(int i=0; i<m.getNbLigne(); i++)
			for(int j=0; j<m.getNbColonne(); j++)
				res.setCase(i, j, coef*m.getCase(i, j));
		return res;
	}
	
	/**
	 * fonction qui calcule la somme de deux matrices de double
	 * @param m1
	 * @param m2
	 * @return
	 */
	static public Matrice<Double> sommeMatrice(Matrice<Double> m1, Matrice<Double> m2)
	{
		Matrice<Double> res = new Matrice<Double>(m1.getNbLigne(),m1.getNbColonne());
		for(int i=0; i<m1.getNbLigne(); i++)
			for(int j=0; j<m1.getNbColonne(); j++)
				res.setCase(i, j, m1.getCase(i, j) + m2.getCase(i, j));
		return res;
	}
	
	

	
	/**
	 * Fonction qui vérifie si deux matrices sont égales.
	 */
	public boolean equals(Matrice<T> m)
	{
		boolean res = true;
		for(int i=0; i<m.getNbLigne(); i++)
			for(int j=0; j<m.getNbColonne(); j++)
				res &= (this.getCase(i, j).equals(m.getCase(i, j)));
		return res;
	}
	
	/**
	 * fonction qui verifie si une matrice appartient a une liste de matrice
	 * @param lm
	 * @return
	 */
	public boolean appartient(ArrayList<Matrice<T>> lm)
	{
		for(int i=0; i<lm.size(); i++)
			if(lm.get(i).equals(this))
				return true;
		return false;
	}
	

	/**
	 * fonction qui retourne la dimension de la matrice
	 * la dimension correspond au nombre de variable libre
	 * par exemple pour une ligne a trois variable la derniere est deduite des deux autres
	 * donc la dimension pour cette ligne est 2
	 * @return
	 */
	public static int Dimension(Matrice<Intervalle> m)
	{
		int dim=0;
		for(int i=0;i<m.getNbLigne();i++)
		{
			for(int j=0;j<m.getNbColonne()-1;j++)
			{
				Intervalle inter = m.getCase(i, j);
				if(!inter.IntervalleUneValeur())
					dim++;
			}
			if(m.getNbColonne()>=2)	
				if(!m.getCase(i, m.getNbColonne()-2).IntervalleUneValeur() && m.getCase(i,m.getNbColonne()-1).IntervalleUneValeur())
					dim--;
		}
		return dim;
	}
	
	/**
	 * fonction qui permet de multiplier une matrice par une liste colonne
	 * 
	 * @param m
	 * @param l
	 * @return
	 */
	public static ArrayList<Double> multListColonne(Matrice<Double> m,ArrayList<Double> l){
		ArrayList<Double> res = new ArrayList<Double>();
		for (int i=0;i<m.getNbLigne();i++)
		{
			double x=0d;
			for(int j=0;j<m.getNbColonne();j++)
				x+=l.get(j)*m.getCase(i, j);
			res.add(x);
		}
		return res;
		}
	
	/**
	 * fonction qui permet de multiplier une liste ligne par une matrice
	 * @param l
	 * @param m
	 * @return
	 */
	public static ArrayList<Double> multListLigne(ArrayList<Double> l,Matrice<Double> m){
		ArrayList<Double> res = new ArrayList<Double>();
		for (int i=0;i<m.getNbColonne();i++)
		{
			double x=0d;
			for(int j=0;j<m.getNbLigne();j++)
				x+=l.get(j)*m.getCase(i, j);
			res.add(x);
		}
		return res;
		}
	
}
