/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package cout;

import calcul.Matrice;
import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;

/**
 * Classe representant les couts pour les requetes concernant les probabilites de passage vers
 * un etat
 * @author Giraudet
 *
 */
public class CoutP2 extends Cout 
{
	private static final long serialVersionUID = 1L;
	private EtatGraphique etat;
	
	
	public CoutP2(String nom, EtatGraphique e, Intervalle ideal, int n)
	{
		super(nom, ideal, n);
		etat = e;
	}

	public EtatGraphique getEtat()
	{
		return etat;
	}
	
	
	@Override
	public boolean estCoutP1() 
	{
		return false;
	}

	@Override
	public boolean estCoutP2() 
	{
		return true;
	}
	
	@Override
	public boolean estCoutP3() 
	{
		return false;
	}

	public int getPosEtatMatrice()
	{
		return etat.getnum()-1;
	}
	
	/**
	 * Permet de générer une matrice de cout
	 * à partir de la matrice en cours
	 */
	public Matrice<Double> genereMatriceCout(Matrice<Double> mp, double val)
	{
		Matrice<Double> res = new Matrice<Double>(mp);
		for(int l=0; l<mp.getNbLigne(); l++)
		{
			if(mp.getCase(l, getPosEtatMatrice()) != null)
				res.setCase(l, getPosEtatMatrice(), mp.getCase(l, getPosEtatMatrice())*val);
		}
		return res;
	}

	@Override
	public boolean estCoutMult() {
		return false;
	}

	
	
}
