/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package dialog;

import ihm.Application;
import ihm.FenetrePrincipale;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.File;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;


/**
 * Classe servant a afficher les courbes obtenues lors d'un calcul de courbe
 * @author giraudet
 *
 */
public class DialogAfficheGraphe extends JDialog{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;





	public DialogAfficheGraphe(String filepng, String fileps)
	{
		super(FenetrePrincipale.getInstance());
		setTitle(filepng);
		this.setLayout(new BorderLayout());
		AffichageImage image = new AffichageImage(filepng);
		add(image,BorderLayout.CENTER);
		JPanel jp = new JPanel();
		File f = new File(fileps);
		JLabel jl = new JLabel(Application.langue.getProperty("tdag")+" "+f.getAbsolutePath());
		jp.add(jl);
		add(jp,BorderLayout.SOUTH);
		setSize(image.largeurEcran,image.hauteurEcran);
		setLocationRelativeTo(FenetrePrincipale.getInstance());
		repaint();
		setVisible(true);
	}





	class AffichageImage extends Canvas
	{
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		int largeurEcran = screenSize.width;
		int hauteurEcran = screenSize.height;
		Image image;

		public AffichageImage(String url)
		{
			image = getToolkit().getImage(url);
			prepareImage(image, this);
		}

		public void paint(Graphics g)
		{
			g.drawImage(image, 0, 0, this);
		}

		public boolean imageUpdate(Image image, int info, int x, int y, int l, int h)
		{
			if ((info & (WIDTH | HEIGHT)) != 0)
			{
				setSize(l, h);
				getParent().getParent().getParent().getParent().setBounds( (largeurEcran - l) / 2, (hauteurEcran - h) / 2, l+8, h+32 );
			}

			if ((info & (ALLBITS)) != 0)
			{
				repaint();
				return false;
			}
			else
			{
				return true;
			}

		}
	}
}