/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package dialog;

import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.dessin.DessinGraphe;
import ihm.dessin.DessinGrapheStateAddCout;
import ihm.intervalle.Intervalle;
import ihm.transition.Transition;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

import util.JFloatField;
import util.JIntField;
import cout.CoutMult;
import cout.CoutP1;

import calcul.Matrice;

/**
 * Dialog utilise dans le cas ou on ajoute une matrice de cout dans le graphe
 * @author giraudet-docet
 *
 */
public class DialogAjoutMatriceCout extends JDialog{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private boolean add;

	public DialogAjoutMatriceCout(JButton valider,JButton annuler,final ArrayList<CoutP1> ListeCout,final ArrayList<CoutMult> lcm,final ArrayList<Transition> ListeTransition,final int nbetat)
	{
		super(FenetrePrincipale.getInstance());
		JPanel jp = new JPanel();
		jp.setLayout(new BoxLayout(jp,BoxLayout.Y_AXIS));
		final JTextField nomcout = new JTextField(20);
		nomcout.setText("C"+(ListeCout.size()+1));
		final JLabel nom = new JLabel(Application.langue.getProperty("nomamc"));
		final JFloatField min = new JFloatField(10);
		final JFloatField max = new JFloatField(10);
		final JIntField fLong = new JIntField(10);
		min.setFloat(0);
		max.setFloat(100);
		fLong.setInt(1);
		JLabel ideal = new JLabel(Application.langue.getProperty("ideal"));
		JLabel _min = new JLabel(Application.langue.getProperty("mininter"));
		JLabel _max = new JLabel(Application.langue.getProperty("maxinter"));
		JLabel _long = new JLabel(Application.langue.getProperty("long"));
		JLabel _choose = new JLabel(Application.langue.getProperty("sortecout"));
		final JRadioButton rMult = new JRadioButton(Application.langue.getProperty("mult"));
		final JRadioButton rAdd = new JRadioButton(Application.langue.getProperty("addi"));
		ButtonGroup type = new ButtonGroup();
		type.add(rMult);
		type.add(rAdd);
		
		
		rAdd.setSelected(true);
		JPanel pnom = new JPanel();
		pnom.add(nom);
		pnom.add(nomcout);
		JPanel ptext = new JPanel();
		ptext.add(ideal);
		JPanel pmin = new JPanel();
		pmin.add(_min);
		pmin.add(min);
		JPanel pmax = new JPanel();
		pmax.add(_max);
		pmax.add(max);
		JPanel pLong = new JPanel();
		pLong.add(_long);
		pLong.add(fLong);
		JPanel pbouton = new JPanel();
		pbouton.add(valider);
		pbouton.add(annuler);
		jp.add(pnom);
		jp.add(ptext);
		jp.add(pmin);
		jp.add(pmax);
		jp.add(pLong);
		JPanel pChoose = new JPanel();
		pChoose.add(_choose);
		jp.add(pChoose);
		JPanel pBoutonRadio = new JPanel();
		pBoutonRadio.add(rAdd);
		pBoutonRadio.add(rMult);
		jp.add(pBoutonRadio);
		JPanel pChoose2 = new JPanel();
		jp.add(pChoose2);
		JPanel pBoutonRadio1 = new JPanel();
		jp.add(pBoutonRadio1);
		jp.add(pbouton);
		add(jp);
		setSize(550,350);
		setResizable(false);
		setLocationRelativeTo(getParent());
		setVisible(true);

		valider.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(min.getFloat()<=max.getFloat())
				{
					String nom = nomcout.getText();
					boolean nomcorrect=true;
					add = rAdd.isSelected();
					if(add)
					for(int i=0;i<ListeCout.size();i++)
					{	
						if(ListeCout.get(i).getNom().compareTo(nom)==0)
						{
							nomcorrect=false;
							break;
						}
							
					}
					else
						for(int i=0;i<lcm.size();i++)
						{	
							if(lcm.get(i).getNom().compareTo(nom)==0)
							{
								nomcorrect=false;
								break;
							}
								
						}
					if(nomcorrect)
					{
						final DessinGraphe dg = FenetrePrincipale.getInstance().getEspaceTravail().getDessinGraphe();
						int n = fLong.getInt();
						if(add)
						{
							Intervalle in = new Intervalle(min.getFloat()/n,max.getFloat()/n);
							Matrice<Integer> Mcout = new Matrice<Integer>(nbetat,nbetat);
						Matrice.MiseAZero(Mcout);
						CoutP1 cout = new CoutP1(nomcout.getText(),Mcout,in,n);
						ListeCout.add(cout);
						dg.setState(new DessinGrapheStateAddCout(dg,cout,true));

						dispose();
						}
						else
						{
							if(n!=0 && min.getFloat()>=0 && max.getFloat()>=0)
							{
							Intervalle in = new Intervalle(Math.pow(min.getFloat(),1d/n),Math.pow(max.getFloat(),1d/n));
							Matrice<Double> Mcout = new Matrice<Double>(nbetat,nbetat);
							Matrice.MiseAUn(Mcout);
							final CoutMult cout = new CoutMult(nomcout.getText(),Mcout,in,n,null);
							JButton valid = new JButton(Application.langue.getProperty("valider"));

							JButton annul = new JButton(Application.langue.getProperty("annuler"));
							new DialogVecteurProba(nbetat,cout,valid,annul);
							valid.addActionListener(new ActionListener()
							{
								public void actionPerformed(ActionEvent e)
								{
									lcm.add(cout);
									dg.setState(new DessinGrapheStateAddCout(dg,cout,false));
								}
							});
							

							dispose();
							}
							else
								JOptionPane.showMessageDialog(getParent(),Application.langue.getProperty("erreurdonnedamc"),null,JOptionPane.INFORMATION_MESSAGE);
						}
						
						
					}


					else
					{
						new ErreurDialog(Application.langue.getProperty("erreurnom"));
					}
				}
				else
				{
					JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(), Application.langue.getProperty("erreurborne"), "ERROR", JOptionPane.ERROR_MESSAGE);
				}
		}
		});

		annuler.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				dispose();
			}
		});

	}

	public boolean isAdditif()
	{
		return add;
	}
	
	
	


}
