/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package dialog;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.chemin.Chemin;
import ihm.dessin.DessinGraphe;
import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.Transition;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import calcul.AccesStateDureeN;
import calcul.Matrice;
import cout.CoutP1;
import util.ConvertArrayToList;
import util.JFloatField;
import util.JIntField;

/**
 * Dialog utilise pour la creation de la requete arrivee a un etat au bout d'une duree n
 * @author giraudet-docet
 *
 */
public class DialogArriveeEtat extends JDialog
{
	private static final long serialVersionUID = 1L;
	private DessinGraphe dg;
	private JList listeEtats;
	private JList listeChemins;

	public DialogArriveeEtat(DessinGraphe g,final boolean chemin)
	{
		super(FenetrePrincipale.getInstance());
		dg = g;
		JPanel jp = new JPanel();
		jp.setLayout(new BoxLayout(jp,BoxLayout.Y_AXIS));
		final JTextField nomcout = new JTextField(20);
		if(!chemin)
		{
			nomcout.setText(Application.langue.getProperty("accesEtat"));
			this.setTitle(Application.langue.getProperty("accesEtat"));
		}
		else
		{
			nomcout.setText(Application.langue.getProperty("accesChemin"));
			this.setTitle(Application.langue.getProperty("accesChemin"));
		}
		final JLabel nom = new JLabel(Application.langue.getProperty("nomamc"));
		final JFloatField min = new JFloatField(10);
		final JFloatField max = new JFloatField(10);
		final JIntField fLong = new JIntField(10);
		min.setFloat(0);
		max.setFloat(100);
		fLong.setInt(1);
		JLabel ideal = new JLabel(Application.langue.getProperty("ideal"));
		JLabel _min = new JLabel(Application.langue.getProperty("mininter"));
		JLabel _max = new JLabel(Application.langue.getProperty("maxinter"));
		JLabel _long = new JLabel(Application.langue.getProperty("long"));
		JPanel pnom = new JPanel();
		pnom.add(nom);
		pnom.add(nomcout);
		JPanel ptext = new JPanel();
		ptext.add(ideal);
		JPanel pmin = new JPanel();
		pmin.add(_min);
		pmin.add(min);
		JPanel pmax = new JPanel();
		pmax.add(_max);
		pmax.add(max);
		JPanel pLong = new JPanel();
		pLong.add(_long);
		pLong.add(fLong);
		
		JPanel pBouton = new JPanel();
		JButton bValide = new JButton(Application.langue.getProperty("ok"));
		JButton bQuit = new JButton(Application.langue.getProperty("quitter"));
		pBouton.add(bValide);
		pBouton.add(bQuit);
		bValide.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(dg.nomDifferentMethode(nomcout.getText()))
				{
					if(min.getFloat()<=max.getFloat())
						{
						Intervalle i = new Intervalle(min.getFloat()/fLong.getInt(),max.getFloat()/fLong.getInt());	
						CoutP1 c;
						if(!chemin)
							c = new CoutP1(nomcout.getText(),calculMatriceCout((EtatGraphique)listeEtats.getSelectedValue()),i,fLong.getInt());
						else
							c = new CoutP1(nomcout.getText(),((Chemin)listeChemins.getSelectedValue()).genereMatriceCout(),i,fLong.getInt());
						dg.methodes.add(new AccesStateDureeN(c,dg.graphe));
						FenetrePrincipale.getInstance().getEspaceTravail().getMenuTraitement().majMethodeResolution();
						dispose();
						}
					else
						new ErreurDialog(Application.langue.getProperty("erreurborne"));
				}
				else
					new ErreurDialog(Application.langue.getProperty("erreurnom"));
			}
		});
		bQuit.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				dispose();
			}
		});
		
		jp.add(pnom);
		jp.add(ptext);
		jp.add(pmin);
		jp.add(pmax);
		jp.add(pLong);
		jp.add(pBouton);
		
		JPanel pTout = new JPanel();
		pTout.setLayout(new BorderLayout());
		if(!chemin)
			pTout.add(listeEtat(),BorderLayout.WEST);
		else
			pTout.add(listeChemin(),BorderLayout.WEST);
		pTout.add(jp,BorderLayout.CENTER);
		
		
		add(pTout);
		setSize(450,300);
		setResizable(false);
		setLocationRelativeTo(getParent());
		setVisible(true);
		
	}
	
	public JPanel listeEtat()
	{
		JPanel res = new JPanel();
		res.setLayout(new BoxLayout(res,BoxLayout.Y_AXIS));
		JLabel lb = new JLabel("States");
		listeEtats = new JList(ConvertArrayToList.convertListChemins(dg.graphe.listeEtats));
		listeEtats.setVisibleRowCount(2);
		listeEtats.setSelectionMode(1);
		listeEtats.setSelectedIndex(0);
		JScrollPane sp = new JScrollPane(listeEtats);
		res.add(lb);
		res.add(sp);
		return res;
	}
	
	public JPanel listeChemin()
	{
		JPanel res = new JPanel();
		res.setLayout(new BoxLayout(res,BoxLayout.Y_AXIS));
		JLabel lb = new JLabel("Paths");
		listeChemins = new JList(ConvertArrayToList.convertListChemins(dg.getChemins()));
		listeChemins.setVisibleRowCount(2);
		listeChemins.setSelectionMode(1);
		listeChemins.setSelectedIndex(0);
		JScrollPane sp = new JScrollPane(listeChemins);
		res.add(lb);
		res.add(sp);
		return res;
	}
	
	
	public Matrice<Integer> calculMatriceCout(EtatGraphique e)
	{
		int N = dg.graphe.getNombreEtats();
		Matrice<Integer> res = new Matrice<Integer>(N,N);
		Matrice.MiseAZero(res);
		
		int c = e.getnum()-1;
		ArrayList<Transition> tEntrantes = e.getTransitionsEntrantes();
		for(int i=0; i<tEntrantes.size(); i++)
		{
			int l = tEntrantes.get(i).getEtatDepart().getnum()-1;
			res.setCase(l, c, 1);
		}
		return res;
	}
	
	
	
	
}
