/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package dialog;

import graphe.Graphe;
import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.transition.RelationProba;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import util.ConvertArrayToList;

/**
 * Dialog utlise pour afficher l'editeur des relations entre les probabilite mise sur les 
 * transitions du graphe
 * @author giraudet-docet
 *
 */
public class DialogListeRelationProba extends JDialog
{
	private static final long serialVersionUID = 1L;
	private Graphe graphe;
	private DialogListeRelationProba me = this;
	private JScrollPane spListeRelation;
	private JList liste1;

	public DialogListeRelationProba(Graphe g)
	{
		super(FenetrePrincipale.getInstance());
		graphe = g;
		construitPanel();
		setTitle(Application.langue.getProperty("nomTitrerp"));
		setLocationRelativeTo(getParent());
		setSize(450, 300);
		setVisible(true);
	}

	public void construitPanel()
	{
		JPanel pTout = new JPanel();
		
		JPanel lp = new JPanel();
		lp.setLayout(new BoxLayout(lp,BoxLayout.Y_AXIS));
		lp.add(new JLabel(Application.langue.getProperty("list")));
		chargeListeRelation();
		lp.add(spListeRelation);
		
		JPanel action = new JPanel();
		action.setLayout(new BoxLayout(action,BoxLayout.Y_AXIS));
		
		JButton bNew = new JButton(Application.langue.getProperty("nouveau"));
		action.add(bNew);
		bNew.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				new DialogRelationProba(me,graphe);
				dispose();
			}
		});
		
		JButton bModif = new JButton(Application.langue.getProperty("modifier"));
		action.add(bModif);
		bModif.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(graphe.listeRelations.size()>0)
				{
					new DialogRelationProba(me,(RelationProba)liste1.getSelectedValue(),graphe);
					dispose();
				}
			}
		});
		
		JButton bSuppr = new JButton(Application.langue.getProperty("supprimer"));
		action.add(bSuppr);
		bSuppr.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(graphe.listeRelations.size()>0)
				{
					graphe.listeRelations.remove(liste1.getSelectedIndex());
					new DialogListeRelationProba(graphe);
					dispose();
				}
			}
		});
		
		JButton bQuit = new JButton(Application.langue.getProperty("quitter"));
		action.add(bQuit);
		bQuit.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				dispose();
			}
		});
		
		
		pTout.add(lp);
		pTout.add(action);
		add(new JScrollPane(pTout));
	}
	
	public void chargeListeRelation()
	{
		liste1 = new JList(ConvertArrayToList.convertListRelations(graphe.listeRelations));
		liste1.setSelectedIndex(0);
		liste1.setVisibleRowCount(10);
		liste1.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		spListeRelation =  new JScrollPane(liste1);
	}
	

	
}
