/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package graphe;

import ihm.chemin.Chemin;
import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.RelationProba;
import ihm.transition.Transition;
import java.awt.Color;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Enumeration;


import att.grappa.Element;
import att.grappa.Graph;
import att.grappa.GrappaPoint;
import att.grappa.Node;
import att.grappa.Parser;
import cout.Cout;
import cout.CoutMult;
import cout.CoutP1;
import calcul.Matrice;
/**
 * Classe servant a representer la structure de donne du graphe
 * @author giraudet-docet
 *
 */
public class Graphe implements Serializable 
{
	private static final long serialVersionUID = 1L;
	public ArrayList<EtatGraphique> listeEtats;
	public transient ArrayList<Transition> listeTransitions;
	protected ArrayList<CoutP1> listeMatriceCoutP1;
	protected ArrayList<CoutMult> lcm;
	private transient Matrice<Double> _resultat;
	protected ArrayList<Chemin> liste_chemin;
	public ArrayList<RelationProba> listeRelations;

	public Graphe()
	{
		listeEtats = new ArrayList<EtatGraphique>();
		listeTransitions = new ArrayList<Transition>();
		listeMatriceCoutP1 = new ArrayList<CoutP1>();
		liste_chemin = new ArrayList<Chemin>();
		listeRelations = new ArrayList<RelationProba>();
		lcm=new ArrayList<CoutMult>();
		_resultat=null;
	}

	//Construction du graphe
	public void ajoutEtat(EtatGraphique e)
	{
		listeEtats.add(e);
		modifieListeCout(listeEtats.indexOf(e));
	}
	
	public void retireEtat(EtatGraphique e)
	{

		modifieListeCoutRemoveEtat(listeEtats.indexOf(e));
		listeEtats.remove(e);
	}
	
	public void supprimeTransition(Transition f)
	{
		
	}

	public void setRayon(int t)
	{
		EtatGraphique.setRayon(t);
	}

	/**
	 * Fonction qui modifie la liste des coutsP1 et coutMult dans le cas d'un ajout d'etat dans le graphe
	 * @param num
	 */
	public void modifieListeCout(int num)
	{
		
		for(int i=0;i<listeMatriceCoutP1.size();i++)
		{
			Matrice<Integer> c = new Matrice<Integer>(num+1,num+1);
			Matrice<Integer> cout = listeMatriceCoutP1.get(i).getMatrice();
			for(int m=0;m<cout.getNbLigne();m++)
				for(int n=0;n<cout.getNbColonne();n++)
					c.setCase(m, n, cout.getCase(m, n));
			for(int n=0;n<c.getNbColonne();n++)
				c.setCase(num, n,0);
			for(int m=0;m<c.getNbLigne();m++)
				c.setCase(m, num,0);
			listeMatriceCoutP1.get(i).setMatrice(c);
		}
		for(int i=0;i<lcm.size();i++)
		{
			Matrice<Double> c = new Matrice<Double>(num+1,num+1);
			Matrice<Double> cout = lcm.get(i).getMatrice();
			for(int m=0;m<cout.getNbLigne();m++)
				for(int n=0;n<cout.getNbColonne();n++)
					c.setCase(m, n, cout.getCase(m, n));
			for(int n=0;n<c.getNbColonne();n++)
				c.setCase(num, n,1d);
			for(int m=0;m<c.getNbLigne();m++)
				c.setCase(m, num,1d);
			lcm.get(i).setMatrice(c);
		}
	}
	
	/**
	 * fonction qui modifie la liste des coutsP1 et coutMult dans le cas ou on retire un
	 * etat dans le graphe
	 * @param num
	 */
	public void modifieListeCoutRemoveEtat(int num)
	{
		for(int i=0;i<listeMatriceCoutP1.size();i++)
		{
			Matrice<Integer> c = new Matrice<Integer>(listeEtats.size()-1,listeEtats.size()-1);
			Matrice<Integer> cout = listeMatriceCoutP1.get(i).getMatrice();
			for(int m=0;m<cout.getNbLigne();m++)
			{
				for(int n=0;n<cout.getNbColonne();n++)
				{
					if((m!=num)&&(n!=num))
						if(m<num)
							if(n<num)
								c.setCase(m,n,cout.getCase(m, n));
							else
								c.setCase(m,n-1,cout.getCase(m, n));
						else
							if(n<num)
								c.setCase(m-1, n, cout.getCase(m, n));
							else
								c.setCase(m-1,n-1,cout.getCase(m, n));
				}
			}
			listeMatriceCoutP1.get(i).setMatrice(c);
		}
		for(int i=0;i<lcm.size();i++)
		{
			Matrice<Double> c = new Matrice<Double>(listeEtats.size()-1,listeEtats.size()-1);
			Matrice<Double> cout = lcm.get(i).getMatrice();
			for(int m=0;m<cout.getNbLigne();m++)
			{
				for(int n=0;n<cout.getNbColonne();n++)
				{
					if((m!=num)&&(n!=num))
						if(m<num)
							if(n<num)
								c.setCase(m,n,cout.getCase(m, n));
							else
								c.setCase(m,n-1,cout.getCase(m, n));
						else
							if(n<num)
								c.setCase(m-1, n, cout.getCase(m, n));
							else
								c.setCase(m-1,n-1,cout.getCase(m, n));
				}
			}
			lcm.get(i).setMatrice(c);
		}
	}
	
	/**
	 * On modifie la liste des couts P1 et Mult dans le cas ou on retire une transition
	 * @param f
	 */
	public void modifieListeCoutTransitionRemoveTransition(Transition f)
	{
		for(int i=0;i<listeMatriceCoutP1.size();i++)
			listeMatriceCoutP1.get(i).getMatrice().setCase(listeEtats.indexOf(f.getEtatDepart()), listeEtats.indexOf(f.getEtatArrivee()),0);
		for(int i=0;i<lcm.size();i++)
			lcm.get(i).getMatrice().setCase(listeEtats.indexOf(f.getEtatDepart()), listeEtats.indexOf(f.getEtatArrivee()),1d);
	}
	
	
	public void ajoutTransition(Transition t)
	{
		listeTransitions.add(t);
		rechercheEtatNum(t.getEtatDepart().getnum()).getTransitionsSortantes().add(t);
		rechercheEtatNum(t.getEtatArrivee().getnum()).getTransitionsEntrantes().add(t);
	}

	public void setColorDefaut()
	{
		for(int i=0;i<listeEtats.size();i++)
			listeEtats.get(i).setColorSelection(null);
		for(int i=0;i<listeTransitions.size();i++)
			listeTransitions.get(i).setColorSel(null);
	}

	public void setColorEtat(Color c)
	{
		EtatGraphique.setColor(c);
	}

	public void setColorTransition(Color c)
	{
		for(int i=0;i<listeTransitions.size();i++)
			listeTransitions.get(i).setColor(c);
	}

	/*public void ajoutRelation(RelationProba rp)
	{
		listeRelations.add(rp);
	}*/


	public Cout rechercheCout(String nom)
	{
		for(int i=0;i<listeMatriceCoutP1.size();i++)
		{
			if(listeMatriceCoutP1.get(i).getNom().compareTo(nom)==0)
				return listeMatriceCoutP1.get(i);
		}
		return null;
	}
	
	public CoutMult rechercheCoutMult(String nom)
	{
		for(int i=0;i<lcm.size();i++)
		{
			if(lcm.get(i).getNom().compareTo(nom)==0)
				return lcm.get(i);
		}
		return null;
	}

	public void SupprimeCout(Cout c)
	{
		listeMatriceCoutP1.remove(c);
	}
	
	public void SupprimeCoutMult(CoutMult c)
	{
		lcm.remove(c);
	}

	public int getNombreEtats()
	{
		return listeEtats.size();
	}

	
	public Matrice<Double> getMatriceResultat()
	{
		return _resultat;
	}
	public ArrayList<CoutP1> getlisteMatriceCoutP1()
	{
		return listeMatriceCoutP1;
	}
	public ArrayList<CoutMult> getlisteCoutMult()
	{
		return lcm;
	}

	public ArrayList<Chemin> getChemins()
	{
		return liste_chemin;
	}
	public ArrayList<Chemin> getCheminsActives()
	{
		ArrayList<Chemin> res = new ArrayList<Chemin>();
		for(int i=0; i<liste_chemin.size(); i++)
			if(liste_chemin.get(i).getActivation())
				res.add(liste_chemin.get(i));
		return res;
	}

	public void setlisteMatriceCoutP1(CoutP1 cout)
	{
		listeMatriceCoutP1.add(cout);
	}
	public void setlisteCoutMult(CoutMult c)
	{
		lcm.add(c);
	}

	public EtatGraphique rechercheEtatNum(int n)
	{
		for (int i=0;i<listeEtats.size();i++)
		{
			if(listeEtats.get(i).getnum()==n)
			{
				return listeEtats.get(i);
			}
		}
		return null;
	}



	/**
	 * Methode qui permet de rechercher un etat a partir de son etiquette
	 * @param eti
	 * @return EtatGraphique
	 */
	public EtatGraphique rechercheEtatParEtiquette(String eti)
	{
		for(int i=0;i<listeEtats.size();i++)
		{
			if(listeEtats.get(i).getEtiquette().compareTo(eti)==0)
			{
				return listeEtats.get(i);
			}
		}
		return null;
	}

	/**
	 * Methode qui permet d'enregistrer une transition
	 * @param f
	 */
	public void enregistreTransition (Transition f)
	{
		if(listeTransitions.indexOf(f)==-1)
		{
			listeTransitions.add(f);
		}
	}

	/**
	 * Fonction qui recherche un etat dans la liste en fonction de la position du clic
	 * @param x abcisse
	 * @param y ordonn
	 * @return etat clique, null sinon
	 */
	public EtatGraphique rechercheEtatGraphique(int x, int y)
	{
		boolean trouve=false;
		int i=0;
		while((i<listeEtats.size())&&(!trouve))
		{
			EtatGraphique j = listeEtats.get(i);
			if(j.estDansEtat(x,y))
			{
				trouve=true;
			}
			else i++;
		}
		if(trouve)
			return listeEtats.get(i);
		else
			return null;
	}

	/**
	 * Fonction qui recherche une Transition dans la liste en fonction de la position du clic
	 * @param x abcisse
	 * @param y ordonn
	 * @return Transition cliqu, null sinon
	 */
	public Transition rechercheTransitionGraphique(int x, int y)
	{	
		boolean trouve=false;
		int i=0;
		while((i<listeTransitions.size())&&(!trouve))
		{
			Transition j = listeTransitions.get(i);
			if(j.estSurTransition(x,y))
			{
				trouve=true;
			}
			else i++;
		}
		if(trouve)
			return listeTransitions.get(i);
		else 
			return null;		
	}


	public void setMatriceResultat(Matrice<Double> m)
	{
		_resultat=m;
	}

	

	/**
	 * fonction qui construit la matrice d'intervalle correspondant aux graphes
	 * les lignes correspondent aux etats de départs
	 * les colonnes au état d'arrivée
	 * @return
	 */
	public Matrice<Intervalle> construireMatriceIntervalle()
	{
		Matrice<Intervalle> mRes = new Matrice<Intervalle>(listeEtats.size(),listeEtats.size());
		for(int i=0; i<listeEtats.size(); i++)
		{
			EtatGraphique e = listeEtats.get(i);
			ArrayList<Transition> l = e.getTransitionsSortantes();
			for(int j=0; j<l.size(); j++)
			{
				Transition t = l.get(j);
				if(t.getIntervalles().size()==0)
					mRes.setCase(listeEtats.indexOf(e), listeEtats.indexOf(t.getEtatArrivee()),  new Intervalle(0,0));
				else
					mRes.setCase(listeEtats.indexOf(e), listeEtats.indexOf(t.getEtatArrivee()), t.getIntervalleProbabilite());
			}
		}
		//on met l'intervalle [0,0] pour les cases de la matrice ou il n'y a pas de transition
		for(int i=0;i<mRes.getNbLigne();i++)
			for(int j=0;j<mRes.getNbColonne();j++)
				if(mRes.getCase(i,j)==null)
					mRes.setCase(i,j,new Intervalle(0,0));
		return mRes;
	}

	/**
	 * fonction qui contruits la liste des matrices de couts
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public ArrayList<Matrice<Integer>> construireCouts()
	{
		ArrayList<Matrice<Integer>> lm = new ArrayList<Matrice<Integer>>();
		for(int i=0;i<listeMatriceCoutP1.size();i++)
			if(listeMatriceCoutP1.get(i).getActivation())
				lm.add(listeMatriceCoutP1.get(i).getMatrice());
		return lm;
	}
	
	/**
	 * fonction qui retourne la liste des couts multiplicatifs actifs
	 */
	public ArrayList<CoutMult> coutMultActif()
	{
		ArrayList<CoutMult> l = new ArrayList<CoutMult>();
		for(int i=0;i<lcm.size();i++)
			if(lcm.get(i).getActivation())
				l.add(lcm.get(i));
		return l;
	}

	/**
	 * fonction qui construit le vecteur idéal
	 * @return
	 */
	public ArrayList<Double> construireIdeal()
	{
		ArrayList<Double> ld = new ArrayList<Double>();
		for(int i=0;i<listeMatriceCoutP1.size();i++)
			if(listeMatriceCoutP1.get(i).getActivation())			
				ld.add(listeMatriceCoutP1.get(i).getIdeal().moyenne());
		ld.addAll(getMoyenneIdealChemin());
		return ld;
	}

	/**
	 * Fonction qui retourne la liste des matrices de couts de chaque chemin
	 */
	@SuppressWarnings("unchecked")
	public ArrayList<Matrice<Integer>> calculCoutsChemin()
	{
		ArrayList<Matrice<Integer>> res = new ArrayList<Matrice<Integer>>();
		for(int i=0; i<liste_chemin.size(); i++)
			if(liste_chemin.get(i).getActivation())
				res.add(liste_chemin.get(i).getCout().getMatrice());
		return res;
	}
	public ArrayList<Double> getMoyenneIdealChemin()
	{
		ArrayList<Double> res = new ArrayList<Double>();
		for(int i=0; i<liste_chemin.size(); i++)
			if(liste_chemin.get(i).getActivation())
				res.add(liste_chemin.get(i).getCout().getIdeal().moyenne());
		return res;
	}


	/**
	 * Fonction qui retourne la transition entre les 2 etats passes en parametre
	 */
	public Transition rechercheTransition(EtatGraphique dep, EtatGraphique arr)
	{
		Transition t = null;
		int cpt = 0;
		while(cpt<listeTransitions.size() && t==null)
		{
			if(listeTransitions.get(cpt).getEtatDepart()!=null && listeTransitions.get(cpt).getEtatArrivee()!=null)
			{
				if(listeTransitions.get(cpt).getEtatDepart().equals(dep) && listeTransitions.get(cpt).getEtatArrivee().equals(arr))
					t = listeTransitions.get(cpt);
			}
			cpt++;
		}
		return t;
	}


	/**
	 * fonction qui affecte a chaque etat sa position lorsqu'elle n'est pas connue
	 * pour cela, on positionne les tats sur un cercle
	 */
	public void definePositionState()
	{
		int rayonMax = getRayonMaxEtat();
		int nbEtat = listeEtats.size();
		int rayon = nbEtat * rayonMax; 
		int centre = rayon+rayonMax+20;
		double angle = (2*Math.PI) / (double)(nbEtat);
		for(int i=0; i<nbEtat; i++)
		{
			listeEtats.get(i).setX(centre + (int)(rayon * Math.cos(angle*i+Math.PI)));
			listeEtats.get(i).setY(centre + (int)(rayon * Math.sin(angle*i+Math.PI)));
		}
	}


	public int getRayonMaxEtat()
	{
		int res = 0;
		for(int i=0; i<listeEtats.size(); i++)
		{
			res = Math.max(res, listeEtats.get(i).getRayon());
		}
		return res;
	}

	/**
	 * fonction qui permet de verifier si une matrice m verifie si elle respecte
	 * les relations sur les probas
	 * @param rl
	 * @param m
	 * @return
	 */
	public static boolean respectRelation(ArrayList<RelationProba> rl, Matrice<Double> m)
	{
		boolean res = true;
		if(rl==null)
			rl = new ArrayList<RelationProba>();
		for(int i=0; i<rl.size(); i++)
		{
			RelationProba rp = rl.get(i);
			Transition p2 = rp.getTransitionP2();
			double valP2 = m.getCase(p2.getEtatDepart().getnum()-1, p2.getEtatArrivee().getnum()-1);
			Transition p1 = rp.getTransitionP1();
			double valP1 = m.getCase(p1.getEtatDepart().getnum()-1, p1.getEtatArrivee().getnum()-1);
			res = res && (valP1 == valP2*rp.getAlpha()+rp.getBeta());
		}
		return res;
	}
	
	

	/**
	 * fonction qui represente une heuristique pour determiner si la matrice m
	 * est proche de respecter les relations de proba
	 * @param lr
	 * @param m
	 * @return
	 */
	public static double evalueMatrice(ArrayList<RelationProba> lr, Matrice<Double> m)
	{
		double res = 0d;
		if(lr==null)
			lr = new ArrayList<RelationProba>();
		for(int i=0; i<lr.size(); i++)
		{
			RelationProba rp = lr.get(i);
			Transition p2 = rp.getTransitionP2();
			double valP2 = m.getCase(p2.getEtatDepart().getnum()-1, p2.getEtatArrivee().getnum()-1);
			Transition p1 = rp.getTransitionP1();
			double valP1 = m.getCase(p1.getEtatDepart().getnum()-1, p1.getEtatArrivee().getnum()-1);
			res += Math.abs(valP1-valP2);
		}
		return res;
	}
	
	

/**
 * fonction qui permet de redessiner le graphe en utilisant la librairie grappa
 *
 */
	public void reorganiserGrappa()
	{
		StringBuilder sBuilder = new StringBuilder ();
		sBuilder.append("digraph G {");
		for(int i=0;i<listeTransitions.size();i++)
		{
			Transition t = listeTransitions.get(i);
			sBuilder.append(" "+t.getEtatDepart().getnum()+" -> "+t.getEtatArrivee().getnum()+";");
		}
		for(int i=0;i<listeEtats.size();i++)
		{
			sBuilder.append("  "+listeEtats.get(i).getnum()+" ");
		}

		Runtime mRuntime = Runtime.getRuntime();
		Process mProc = null;
		try {
			mProc = mRuntime.exec("dot");
		} catch (IOException ex) {
			ex.printStackTrace();
		}

		DataOutputStream dataoutputstream = new DataOutputStream(mProc.getOutputStream());
		try {
			dataoutputstream.writeBytes(sBuilder.toString());
			dataoutputstream.flush();
			dataoutputstream.close();
		} catch (IOException ex) {
			ex.printStackTrace();
		}  

		InputStream inputstream = mProc.getInputStream();
		Parser parser = new Parser (inputstream);
		try {
			parser.parse();
		} catch (Exception e) {
			System.err.println(e);
			System.exit(0);
		}

		Graph graph = parser.getGraph();
		for (Enumeration e = graph.nodeElements() ; e.hasMoreElements() ;) {
			Element elem = (Element)e.nextElement();
			String nom = elem.toString();
			EtatGraphique et = this.rechercheEtatNum(new Integer(nom));
			GrappaPoint gp = ((Node)(elem)).getCenterPoint();
			et.setX((int)gp.x*2);
			et.setY(-(int)gp.y*2);

		} 
		
		sBuilder.append(" }");

	}
	
	/**
	 * fonction qui permet de passer le graphe en format dot
	 * @return
	 */
	public String toDot()
	{
		String res = "";
		res += "digraph G { \n";
		for(int i=0;i<listeTransitions.size();i++)
		{
			Transition t = listeTransitions.get(i);
			res += t.getEtatDepart().getnum()+" -> "+t.getEtatArrivee().getnum()+";\n";
		}
		for(int i=0;i<listeEtats.size();i++)
		{
			res += listeEtats.get(i).getnum()+"\n";
		}
		res += "}";
		return res;
	}

	public String toString()
	{
		return "Etats : " + listeEtats + "\n" + "Transitions : " + listeTransitions;
	}


}
