/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm;

import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.Transition;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import graphe.Graphe;
/**
 * Classe qui permet de charger un fichier texte produit  par le logiciel prism
 * @author giraudet-docet
 *
 */
public class ChargementFichierGraphe 
{
	private String filePath;
	private Graphe graphe;
	private FenetrePrincipale fenetre;
	
	public ChargementFichierGraphe(FenetrePrincipale fp)
	{
		filePath = getFile(fp);
		fenetre = fp;
		traitementFichier();
	}
	
	public Graphe chargeGraphe()
	{
		return graphe;
	}
	
	public void traitementFichier()
	{
		if(filePath!=null)
		{
			graphe = new Graphe();
			BufferedReader fic = null;
			String ligne;
			try
			{
				fic = new BufferedReader(new FileReader(filePath));
				while ((ligne = fic.readLine()) != null)
				{
					traiteEtat(ligne);
				}
				fic.close();
				fic = new BufferedReader(new FileReader(filePath));
				while ((ligne = fic.readLine()) != null)
				{
					traiteTransition(ligne);
				}
				fic.close();
				ajoutIntervalle();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
	}
	
	public void traiteEtat(String ligne)
	{
		ligne = supprimeEspace(ligne);
		StringTokenizer st = new StringTokenizer(ligne,"|");
		if(st.hasMoreTokens())
		{
			String s = st.nextToken();
			if(isNumber(s.charAt(0)))
			{
				graphe.ajoutEtat(new EtatGraphique(s));
			}
		}
	}
	public void traiteTransition(String ligne)
	{
		ligne = supprimeEspace(ligne);
		if(ligne.length()>0 && isNumber(ligne.charAt(0)))
		{
			StringTokenizer st = new StringTokenizer(ligne,"|");
			if(st.countTokens()==4)
			{
				String s="";
				String etat = st.nextToken();
				while(st.hasMoreTokens())
					s = st.nextToken();
				ArrayList<String> ls = getListeTransition(s);
				if(ls.size()==0)
					graphe.ajoutTransition(new Transition(graphe.rechercheEtatParEtiquette(etat),graphe.rechercheEtatParEtiquette(etat)));
				else
					for(int i=0; i<ls.size(); i++)
						graphe.ajoutTransition(new Transition(graphe.rechercheEtatParEtiquette(etat),graphe.rechercheEtatParEtiquette(convertToTransition(ls.get(i)))));
			}
		}
	}

	public ArrayList<String> getListeTransition(String s)
	{
		ArrayList<String> ls = new ArrayList<String>();
		s = s.substring(1, s.length()-1);
		StringTokenizer st = new StringTokenizer(s,"[");
		while(st.hasMoreTokens())
			ls.add(st.nextToken());
		return ls;
	}
	
	
	public String convertToTransition(String s)
	{
		String res = s.replace("[", "");
		res = res.replace(",","");
		res = res.replace("]","");
		return res;
	}
	
	public String supprimeEspace(String s)
	{
		return s.replace(" ", "");
	}
	
	@SuppressWarnings("static-access")
	public String getFile(FenetrePrincipale fp)
	{
		JFileChooser fc = new JFileChooser();
		MonFiltre filtre = new MonFiltre("txt");
		fc.setFileFilter(filtre);
		fc.showOpenDialog(fenetre);
		File f;
		f=fc.getSelectedFile();
		if(f != null)
		{
			String s=f.getPath();
			String suf=new String();
			int i = s.lastIndexOf('.');
			if (i > 0 &&  i < s.length() - 1) 
			{
				suf = s.substring(i+1).toLowerCase();
				if(suf.compareTo("txt")!=0)
				{
					JOptionPane.showMessageDialog(fenetre, Application.langue.getProperty("erreurtxt"), "ERROR", JOptionPane.ERROR_MESSAGE);
					return null;
				}
				else{
					fp.setTitle(fp.NOM_LOGICIEL+ " - "+f.getName());
					return f.getPath();
				}
			}
			else
			{
				JOptionPane.showMessageDialog(fenetre, Application.langue.getProperty("erreurtxt"), "ERROR", JOptionPane.ERROR_MESSAGE);
				return null;
			}
		}
		return null;
	}
	
	public static boolean isNumber(char c)
	{
		return (c=='0' || c=='1' || c=='2' || c=='3' || c=='4' || c=='5' || c=='6' || c=='7' || c=='8' || c=='9');
	}
	
	public void ajoutIntervalle()
	{
		for(int i=0; i<graphe.getNombreEtats(); i++)
		{
			ArrayList<Transition> lt = graphe.listeEtats.get(i).getTransitionsSortantes();
			if(lt.size()==1)
				lt.get(0).addIntervalle(new Intervalle(100,100));
			else
				for(int j=0;j<lt.size();j++)
					lt.get(j).addIntervalle(new Intervalle(0,100));			
		}
	}
	
}
