/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm;

import graphe.Graphe;
import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.Transition;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.StringTokenizer;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import att.grappa.Edge;
import att.grappa.Graph;
import att.grappa.Node;
import att.grappa.Parser;
/**
 * Classe servant a importer un fichier dot
 * @author giraudet-docet
 *
 */
public class ImportationDot 
{
	private File file;
	private FenetrePrincipale fenetre;
	private FileInputStream input;
	private boolean aPosition;

	@SuppressWarnings("static-access")
	public ImportationDot(FenetrePrincipale fp)
	{
		fenetre = fp;
		String path = getFile();
		if(path!=null)
		{
			
			file = new File(path);
			fp.setTitle(fp.NOM_LOGICIEL+ " - "+file.getName());
			try 
			{
				input = new FileInputStream(file);
			} 
			catch (FileNotFoundException e) {
				e.printStackTrace();
			}
		}
	}

	public Graphe chargeGraphe()
	{
		if(file!=null)
		{
			Parser pars = new Parser(input);
			try 
			{
				pars.parse();
			} 
			catch (Exception e){
				e.printStackTrace();
			}

			Graph grap = pars.getGraph();

			return graphToGraphe(grap);
		}
		else
			return null;
	}


	public Graphe graphToGraphe(Graph g)
	{
		Graphe res = new Graphe();
		aPosition = true;
		Enumeration es = g.subgraphElements();
		if(es.hasMoreElements())
		{
			JOptionPane.showMessageDialog(fenetre,Application.langue.getProperty("importDotErreur"), "ERROR", JOptionPane.ERROR_MESSAGE);
			return null;
		}
		else
		{
			for (Enumeration e = g.nodeElements() ; e.hasMoreElements() ;) 
			{
				Node elem = (Node)e.nextElement();

				String pos = elem.getAttribute("pos").getValue().toString();
				StringTokenizer st = new StringTokenizer(pos,",");
				int x = (Double.valueOf(st.nextToken())).intValue();
				int y = -(Double.valueOf(st.nextToken())).intValue();
				aPosition &= (x!=0 && y!=0);

				EtatGraphique etat = new EtatGraphique(getEtiquetteEtat(elem));
				etat.setX(x);
				etat.setY(y);
				res.ajoutEtat(etat);
			}

			for(Enumeration e = g.edgeElements(); e.hasMoreElements(); )
			{
				Edge elem = (Edge)e.nextElement();

				EtatGraphique arr = res.rechercheEtatParEtiquette(getEtiquetteEtat(elem.getHead()));
				EtatGraphique dep = res.rechercheEtatParEtiquette(getEtiquetteEtat(elem.getTail()));
				Transition t = new Transition(dep,arr);
				res.ajoutTransition(t);
			}

			if(!aPosition)
				//res.definePositionState();
				res.reorganiserGrappa();
			
			for(int i=0; i<res.getNombreEtats(); i++)
			{
				ArrayList<Transition> lt = res.listeEtats.get(i).getTransitionsSortantes();
				if(lt.size()==1)
					lt.get(0).addIntervalle(new Intervalle(100,100));
				else
					for(int j=0;j<lt.size();j++)
						lt.get(j).addIntervalle(new Intervalle(0,100));			
			}
			return res;
		}
	}

	public static String getEtiquetteEtat(Node etat)
	{
		String eti = etat.getAttribute("label").getValue().toString();
		if(eti.startsWith("\\"))
			eti = "state"+etat.getName();
		return eti;
	}


	public String getFile()
	{
		JFileChooser fc = new JFileChooser();
		MonFiltre filtre = new MonFiltre("dot");
		fc.setFileFilter(filtre);
		fc.showOpenDialog(fenetre);
		File f;
		f=fc.getSelectedFile();
		if(f != null)
		{
			String s=f.getPath();
			String suf=new String();
			int i = s.lastIndexOf('.');
			if (i > 0 &&  i < s.length() - 1) 
			{
				suf = s.substring(i+1).toLowerCase();
				if(suf.compareTo("dot")!=0)
				{
					JOptionPane.showMessageDialog(fenetre, Application.langue.getProperty("erreurdot"), "ERROR", JOptionPane.ERROR_MESSAGE);
					return null;
				}
				else return f.getPath();
			}
			else
			{
				JOptionPane.showMessageDialog(fenetre, Application.langue.getProperty("erreurdot"), "ERROR", JOptionPane.ERROR_MESSAGE);
				return null;
			}
		}
		return null;
	}

}
