/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm;

import ihm.chemin.Chemin;
import ihm.dessin.DessinGraphe;
import ihm.dessin.DessinGrapheStateAddChemin;
import ihm.dessin.DessinGrapheStateAddCout;
import ihm.dessin.DessinGrapheStateAddIntervalle;
import ihm.dessin.DessinGrapheStateAddTransition;
import ihm.dessin.DessinGrapheStateDefault;
import ihm.dessin.DessinGrapheStateSuppression;
import ihm.etat.EtatGraphique;
import ihm.etat.EtatGraphiqueFinal;
import ihm.etat.EtatInitialGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.Transition;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import thread.ThreadCalcul;
import util.ConvertArrayToList;
import calcul.Matrice;
import calcul.MethodeResolution;
import cout.Cout;
import cout.CoutP1;
import cout.CoutMult;
import dialog.CheminIncorrect;
import dialog.DialogAfficheChemin;
import dialog.DialogAfficheCout;
import dialog.DialogAjoutMatriceCout;
import dialog.DialogModifieCout;
import dialog.DialogRechercheLocale;
import dialog.ErreurDialog;
import dialog.ModificationEtat;
import dialog.ModificationTransition;
/**
 * Classe qui herite de la classe JPanel et affiche le menu du dessin
 * @author Docet Giraudet
 *
 */
public class MenuTraitement  extends JPanel 
{
	static final long serialVersionUID = 0;
	static final String TITRE_CADRE = "Tools";
	protected DessinGraphe espaceDessin;
	protected ArrayList liste_jpanel=new ArrayList(10);
	private transient MenuTraitement mt = this;
	
	//Descriptions des différents outils
	protected JButton bAjoutEtat = new JButton(Application.langue.getProperty("bAjoutEtat"));
	protected JButton bAjoutTransition = new JButton(Application.langue.getProperty("bAjoutTransition"));
	protected JButton bAjoutIntervalle = new JButton(Application.langue.getProperty("bAjoutIntervalle"));
	protected JButton bSuppression = new JButton(Application.langue.getProperty("bSuppression"));
	protected JButton bAjoutChemin = new JButton(Application.langue.getProperty("bAjoutChemin"));
	protected JButton bAjoutCout = new JButton(Application.langue.getProperty("bAjoutCout"));

	//Different Panel
	protected JPanel pBoutons;
	protected JPanel pChemins;
	protected JPanel pConsultation;
	protected JPanel pCout;
	protected JPanel pMethodeResolution;
	protected JPanel pThread;
	
	private boolean creationCheminEnCours = false;
	private boolean creationCoutEnCours = false;
	/**
	 * Constructeur de la classe au quel on lui donne l'instance de la 
	 * classe DessinGraphe 
	 * @param dg DessinGraphe
	 */
	public MenuTraitement(DessinGraphe dg)
	{
		espaceDessin = dg;
		setBorder(BorderFactory.createTitledBorder(TITRE_CADRE));
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		ajoutElementsGraphe();
		consulteElementsGraphe();
		afficheChemins();
		afficheCout();
		afficheMethodeResolution();
		afficheThread();
		add(pBoutons);	
		add(pChemins);
		add(pCout);
		add(pMethodeResolution);
		add(pThread);
		add(pConsultation);
	}
	
	
	
	/**
	 * permet de construire de facon indépendante les boutons d'ajout dans le graphe
	 * @return le JPanel contenant les boutons d'ajout dans le graphe
	 */
	protected void ajoutElementsGraphe()
	{
		pBoutons = new JPanel();
		pBoutons.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pBoutonsMenu")));
		GridLayout gl = new GridLayout(3,2);
		gl.setHgap(2);
		gl.setVgap(2);
		JPanel pBout = new JPanel();
		pBout.setLayout(gl);
		pBout.add(bAjoutEtat);
		pBout.add(bAjoutTransition);
		pBout.add(bSuppression);
		pBout.add(bAjoutIntervalle);
		pBout.add(bAjoutChemin);
		pBout.add(bAjoutCout);
		pBoutons.add(pBout);
		
		bAjoutEtat.addActionListener(new ActionListener()
		{
			@SuppressWarnings("unchecked")
			public void actionPerformed(ActionEvent e)
			{
				final JDialog j = new JDialog(FenetrePrincipale.getInstance());
				j.setResizable(false);
				
				final JRadioButton rNormal = new JRadioButton(Application.langue.getProperty("normal"));
				JRadioButton rInitial = new JRadioButton(Application.langue.getProperty("initial"));
				final JRadioButton rFinal = new JRadioButton(Application.langue.getProperty("final"));
				ButtonGroup type = new ButtonGroup();
				type.add(rNormal);
				type.add(rInitial);
				type.add(rFinal);
				rNormal.setSelected(true);
				
				final JTextField etiquette = new JTextField(20);
				etiquette.setText(Application.langue.getProperty("etat")+(espaceDessin.graphe.getNombreEtats()+1));
				JLabel eti = new JLabel(Application.langue.getProperty("eti"));
				JPanel p1 = new JPanel();
				JPanel p2 = new JPanel();
				JPanel p3 = new JPanel();

				JButton creer = new JButton(Application.langue.getProperty("creer"));
				p1.setLayout(new FlowLayout());
				p1.add(eti);
				p1.add(etiquette);
				
				p2.setLayout(new FlowLayout());
				p2.add(rNormal);
				p2.add(rInitial);
				p2.add(rFinal);
				
				p3.setLayout(new FlowLayout());
				p3.add(creer);
				j.setLayout(new BorderLayout());
				j.add(p2,BorderLayout.CENTER);
				j.add(p1,BorderLayout.NORTH);
				j.add(p3,BorderLayout.SOUTH);
				j.setSize(400,150);
				j.setLocationRelativeTo(FenetrePrincipale.getInstance());
				j.setVisible(true);
				
				creer.addActionListener(new ActionListener()
						{
					public void actionPerformed(ActionEvent e)
					{
						EtatGraphique etat;
						//String s = (String) jcb.getSelectedItem();
						String et = etiquette.getText();
						if (contrainteEtiquetteEtatDifferent(et))
						{
							if(rNormal.isSelected())
							etat = new EtatGraphique(et);
							else if (rFinal.isSelected())
								etat = new EtatGraphiqueFinal(et);
							else
							etat = new EtatInitialGraphique(et);
							j.dispose();
							espaceDessin.setLayout(null);//on place les etats en fonction de leur coordonnees
							espaceDessin.add(etat);
						
							espaceDessin.addetat(etat);
						
							etat.setOpaque(false);//permet de voir l arriere plan
							espaceDessin.repaint();//redessine tout
						}
						else
						{
							new ErreurDialog(Application.langue.getProperty("erreurEtat"));
							j.dispose();
						}
					}
						});
				
			}
		});
		
		bAjoutTransition.addActionListener(new ActionListener() 
                { 
                    public void actionPerformed(ActionEvent e) 
                    { 
                    	espaceDessin.setState(new DessinGrapheStateAddTransition(espaceDessin));
                    } 
             
                });
		
		bSuppression.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
					
						espaceDessin.setState(new DessinGrapheStateSuppression(espaceDessin));
					}
				});
		
		bAjoutIntervalle.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						
						espaceDessin.setState(new DessinGrapheStateAddIntervalle(espaceDessin));
						
					}			
				});
		bAjoutChemin.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						if(!creationCheminEnCours)
						{
							//System.out.println("Creation d'un chemin");
							creationCheminEnCours = true;
							enabledChemin(true);
							espaceDessin.setState(new DessinGrapheStateAddChemin(espaceDessin));
						}
						else
						{
							//System.out.println("Validation d'un chemin");
							if(!espaceDessin.getChemins().get(espaceDessin.getChemins().size()-1).finPossible())
							{
								new CheminIncorrect(espaceDessin.getChemins(),mt);
							}
							else
							{
								terminerConstructionChemin();
							}
						
						}
					}			
				});
		
		bAjoutCout.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						if(!creationCoutEnCours)
						{
							
							JButton valider = new JButton(Application.langue.getProperty("ok"));
							JButton annuler = new JButton(Application.langue.getProperty("annuler"));
							enabledCout(true);
							final DialogAjoutMatriceCout damc = new DialogAjoutMatriceCout(valider,annuler,espaceDessin.graphe.getlisteMatriceCoutP1(),espaceDessin.graphe.getlisteCoutMult(),espaceDessin.getListeTransition(),espaceDessin.graphe.getNombreEtats());
							damc.addWindowListener(new WindowAdapter(){
								public void windowClosing(WindowEvent e){
									creationCoutEnCours=false;
									enabledCout(false);
								}	
							});
							valider.addActionListener(new ActionListener()
									{
								public void actionPerformed(ActionEvent e)
								{
									
							
									creationCoutEnCours=true;
								}
									});
							annuler.addActionListener(new ActionListener()
							{
								public void actionPerformed(ActionEvent e)
								{
									creationCoutEnCours=false;
									espaceDessin.setState(new DessinGrapheStateDefault(espaceDessin));
								enabledCout(false);
								}
							});
							
						}
						else
						{
							majCout();
							espaceDessin.setState(new DessinGrapheStateDefault(espaceDessin));
							enabledCout(false);
							creationCoutEnCours=false;
						}
					}
				});
	}
	
	/**
	 * fonction qui active ou desactive les boutons lors d'un ajout chemin
	 * @param x
	 */
	public void enabledChemin(boolean x)
	{
			bAjoutEtat.setEnabled(!x);
			bAjoutTransition.setEnabled(!x);
			bAjoutIntervalle.setEnabled(!x);
			bSuppression.setEnabled(!x);
			bAjoutCout.setEnabled(!x);
			bAjoutChemin.setSelected(x);
	}
	
	public void setCheminEnCours(boolean x)
	{
		creationCheminEnCours=x;
	}
	
	/**
	 * fonction qui active ou desactive les boutons lors d'un ajout cout
	 * @param x
	 */
	public void enabledCout(boolean x)
	{
			bAjoutEtat.setEnabled(!x);
			bAjoutTransition.setEnabled(!x);
			bAjoutIntervalle.setEnabled(!x);
			bSuppression.setEnabled(!x);
			bAjoutChemin.setEnabled(!x);
			bAjoutCout.setSelected(x);
	}
	
	public void terminerConstructionChemin()
	{
		majChemins();
		espaceDessin.setState(new DessinGrapheStateDefault(espaceDessin));
		creationCheminEnCours = false;
		enabledChemin(false);
	}

	
	/**
	 * permet de construire les divers éléments de consultation du graphe
	 * @return le JPanel contenant tous les éléments nécessaires
	 */
	protected void consulteElementsGraphe()
	{
		pConsultation = new JPanel();
		pConsultation.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pConsultMenu")));
		
	}
	
	/**
	 * fonction utiliser lorsque que l'on clique sur une transition
	 * @param pn
	 * @param t
	 */
	public void setConsultationTransition(JPanel pn, final Transition t)
	{
		pConsultation.removeAll();
		if(pn!=null)
		{
			pConsultation.setLayout(new GridLayout(2,1));
			pConsultation.add(pn);
			//Bouton de modification d'un �tat
			JButton bModif = new JButton(Application.langue.getProperty("modifier"));
			final JButton bQuitterModif = new JButton(Application.langue.getProperty("valider"));
			bModif.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e)
				{
					new ModificationTransition(t,bQuitterModif);
				}
			});
			bQuitterModif.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e)
				{
					repaint();
				}
			});
			JPanel boutons = new JPanel();
			boutons.add(bModif);
			pConsultation.add(boutons);
		}
		repaint();
		setVisible(false);
		setVisible(true);
	}
	
	/**
	 * fonction utiliser lorque l'on clique sur un etat
	 * @param pn
	 * @param eg
	 */
	public void setConsultationEtat(JPanel pn, final EtatGraphique eg)
	{
		pConsultation.removeAll();
		if(pn!=null)
		{
			pConsultation.setLayout(new GridLayout(2,1));
			pConsultation.add(pn);
			//Bouton de modification d'un �tat
			JButton bModif = new JButton(Application.langue.getProperty("modifier"));
			final JButton bValider = new JButton(Application.langue.getProperty("ok"));
			bModif.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e)
				{
					new ModificationEtat(eg,bValider);
				}
			});
			bValider.addActionListener(new ActionListener(){
				public void actionPerformed(ActionEvent e)
				{
					repaint();
				}
			});
			JPanel boutons = new JPanel();
			boutons.add(bModif);
			pConsultation.add(boutons);
		}
		repaint();
		setVisible(false);
		setVisible(true);
	}
	
	/**
	 * fonction utiliser lorsque l'on clique sur rien
	 *
	 */
	public void setConsultationVide()
	{
		pConsultation.removeAll();
		repaint();
		setVisible(false);
		setVisible(true);
	}
	
	
	protected void afficheCout()
	{
		pCout = new JPanel();
		pCout.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pCout")));
		majCout();
	}
	
	/**
	 * fonction qui permet de faire la mise a jour du panel contenant la liste des couts
	 *
	 */
	public void majCout()
	{
		pCout.removeAll();
		ArrayList<Cout> lc = new ArrayList<Cout>();
		lc.addAll(espaceDessin.getListeCout());
		lc.addAll(espaceDessin.graphe.getlisteCoutMult());
		final JList liste = new JList(ConvertArrayToList.convertListCout(lc));
		liste.setVisibleRowCount(3);
		liste.setSelectionMode(1);
		liste.setSelectedIndex(0);
		JScrollPane sp = new JScrollPane(liste);
		JButton bAffiche = new JButton(Application.langue.getProperty("afficher"));
		bAffiche.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				if(liste.getSelectedValue()!=null)
				{
					if(((Cout)liste.getSelectedValue()).estCoutP1() || ((Cout)liste.getSelectedValue()).estCoutMult())
						new DialogAfficheCout(((Cout)liste.getSelectedValue()));
					else
					{/*cas ou le cout est de type P2*/}
					}
			}
		});
		JButton bModifie = new JButton(Application.langue.getProperty("modifier"));
		bModifie.addActionListener(new ActionListener()
		{
			@SuppressWarnings("unchecked")
			public void actionPerformed(ActionEvent e)
			{

				if(liste.getSelectedValue()!=null)
				{
					Cout c2 = (Cout)liste.getSelectedValue();
					if(c2.estCoutP1())
					{
						final CoutP1 c = (CoutP1)c2;
						Matrice<Integer> m1 = new Matrice(c.getMatrice());
						String nom = new String(c.getNom());
						Intervalle in = c.getIdeal();
						final CoutP1 c1 = new CoutP1(nom,m1,in,c.getDuree());

						JButton valider = new JButton(Application.langue.getProperty("ok"));
						JButton annuler = new JButton(Application.langue.getProperty("annuler"));
						valider.addActionListener(new ActionListener()
						{
							public void actionPerformed(ActionEvent e)
							{
								espaceDessin.setState(new DessinGrapheStateDefault(espaceDessin));
								majCout();
							}
						});
						annuler.addActionListener(new ActionListener()
						{
							public void actionPerformed(ActionEvent e)
							{
								c.setNom(c1.getNom());
								c.setMatrice(c1.getMatrice());
								c.setIdeal(c1.getIdeal());
								majCout();
								espaceDessin.setState(new DessinGrapheStateDefault(espaceDessin));
							}
						});
						espaceDessin.setState(new DessinGrapheStateAddCout(espaceDessin,c,true));
						new DialogModifieCout(c,valider,annuler);
					}
					else
					{
						final CoutMult c3 = (CoutMult)c2;
						Matrice<Double> m1 = new Matrice(c3.getMatrice());
						String nom = new String(c3.getNom());
						Intervalle in = c3.getIdeal();
						final CoutMult c4 = new CoutMult(nom,m1,in,c3.getDuree(),c3.getVecteurProba());

						JButton valider = new JButton(Application.langue.getProperty("ok"));
						JButton annuler = new JButton(Application.langue.getProperty("annuler"));
						valider.addActionListener(new ActionListener()
						{
							public void actionPerformed(ActionEvent e)
							{
								espaceDessin.setState(new DessinGrapheStateDefault(espaceDessin));
								majCout();
							}
						});
						annuler.addActionListener(new ActionListener()
						{
							public void actionPerformed(ActionEvent e)
							{
								c3.setNom(c4.getNom());
								c3.setMatrice(c4.getMatrice());
								c3.setIdeal(c4.getIdeal());
								c3.setVecteurProba(c4.getVecteurProba());
								majCout();
								espaceDessin.setState(new DessinGrapheStateDefault(espaceDessin));
							}
						});
						espaceDessin.setState(new DessinGrapheStateAddCout(espaceDessin,c3,false));
						new DialogModifieCout(c3,valider,annuler);
					}	

				}

			}

		});
		JButton bSupprime = new JButton(Application.langue.getProperty("supprimer"));
		bSupprime.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(liste.getSelectedValue()!=null)
				{
					if(((Cout)liste.getSelectedValue()).estCoutP1())
						espaceDessin.SupprimeCout((Cout)liste.getSelectedValue());
					else
						espaceDessin.graphe.SupprimeCoutMult((CoutMult)liste.getSelectedValue());
					majCout();
				}
			}
		});

		JPanel boutons = new JPanel();
		boutons.add(bAffiche);
		boutons.add(bModifie);
		boutons.add(bSupprime);
		
		final JCheckBox check = new JCheckBox();
		check.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				((Cout)liste.getSelectedValue()).setActivation(check.isSelected());
			}
		});
		
		liste.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e) 
			{
				check.setSelected(((Cout)liste.getSelectedValue()).getActivation());
			}
		});
		
		if(liste.getSelectedValue()!=null)
		{
			check.setSelected(((Cout)liste.getSelectedValue()).getActivation());
			boutons.add(check);
		}
		
		pCout.setLayout(new GridLayout(2,1));
		pCout.add(sp);
		pCout.add(boutons);
		setVisible(false);
		setVisible(true);
		repaint();
	}
	
	/**
	 * Permet de consulter tous les chemins enregistés jusque la
	 */
	protected void afficheChemins()
	{
		pChemins = new JPanel();
		pChemins.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pPath")));
		majChemins();
	}
	
	/**
	 * fonction qui met a jour le panel contenant la liste des chemins
	 *
	 */
	public void majChemins()
	{
		pChemins.removeAll();
		final JList liste = new JList(ConvertArrayToList.convertListChemins(espaceDessin.getChemins()));
		liste.setVisibleRowCount(3);
		liste.setSelectionMode(1);
		liste.setSelectedIndex(0);
		JScrollPane sp = new JScrollPane(liste);
		
		JButton bAffiche = new JButton(Application.langue.getProperty("afficher"));
		bAffiche.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(liste.getSelectedValue()!=null)
				{
					new DialogAfficheChemin((Chemin)liste.getSelectedValue());
					espaceDessin.afficheChemin((Chemin)liste.getSelectedValue());
				}
				
			}
		});
		JButton bSupprime = new JButton(Application.langue.getProperty("supprimer"));
		bSupprime.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(liste.getSelectedValue()!=null)
				{
					espaceDessin.getChemins().remove((Chemin)liste.getSelectedValue());
					espaceDessin.afficheChemin((Chemin)liste.getSelectedValue());
					majChemins();
				}
			}
		});
		
		JPanel boutons = new JPanel();
		boutons.add(bAffiche);
		boutons.add(bSupprime);
		
		final JCheckBox check = new JCheckBox();
		check.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				((Chemin)liste.getSelectedValue()).setActivation(check.isSelected());
			}
		});
		
		liste.addListSelectionListener(new ListSelectionListener()
		{
			public void valueChanged(ListSelectionEvent e) 
			{
				check.setSelected(((Chemin)liste.getSelectedValue()).getActivation());
			}
		});
		
		if(liste.getSelectedValue()!=null)
		{
			check.setSelected(((Chemin)liste.getSelectedValue()).getActivation());
			boutons.add(check);
		}
		
		
		pChemins.setLayout(new GridLayout(2,1));
		pChemins.add(sp);
		pChemins.add(boutons);
		setVisible(false);
		setVisible(true);
		repaint();
	}
	
	
	protected void afficheMethodeResolution()
	{
		pMethodeResolution = new JPanel();
		pMethodeResolution.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pRequete")));
		majMethodeResolution();
	}
	
	/**
	 * fonction qui met a jour le panel contenant toutes les methodes de resolution
	 *
	 */
	public void majMethodeResolution()
	{
		pMethodeResolution.removeAll();
		final JList liste = new JList(ConvertArrayToList.convertListMethode(espaceDessin.methodes));
		liste.setVisibleRowCount(3);
		liste.setSelectionMode(1);
		liste.setSelectedIndex(0);
		JScrollPane sp = new JScrollPane(liste);
		
		JButton bExec = new JButton(Application.langue.getProperty("executer"));
		final JButton bSupprime = new JButton(Application.langue.getProperty("supprimer"));
		bExec.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				new DialogRechercheLocale(espaceDessin,bSupprime);
			}
		});
		
	

		
		bSupprime.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(liste.getSelectedValue()!=null)
				{
					if(liste.getSelectedIndex() != 0 || liste.getSelectedIndex() != 1)
						espaceDessin.methodes.remove((MethodeResolution)liste.getSelectedValue());
 					majMethodeResolution();
				}
			}
		});
		if((espaceDessin.threadCalculs.size()==0)&&(FenetrePrincipale.getInstance().getLTC().size()==0))
			bSupprime.setEnabled(true);
		JPanel boutons = new JPanel();
		boutons.add(bExec);
		boutons.add(bSupprime);
		
		pMethodeResolution.setLayout(new GridLayout(2,1));
		pMethodeResolution.add(sp);
		pMethodeResolution.add(boutons);
		setVisible(false);
		setVisible(true);
		repaint();
	}
	
	public void afficheThread()
	{
		pThread=new JPanel();
		pThread.setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("pThread")));
		majThread();
	}
	
	/**
	 * fonction qui met a jour le panel contenant tout les calculs lances
	 *
	 */
	public void majThread()
	{
		pThread.removeAll();
		ArrayList<ThreadCalcul> ltc = new ArrayList<ThreadCalcul>();
		ltc.addAll(espaceDessin.threadCalculs);
		ltc.addAll(FenetrePrincipale.getInstance().getLTC());
		final JList liste = new JList(ConvertArrayToList.convertListThreadCalcul(ltc));
		liste.setVisibleRowCount(3);
		liste.setSelectionMode(1);
		liste.setSelectedIndex(0);
		JScrollPane sp = new JScrollPane(liste);
		
		JButton voirResultat = new JButton(Application.langue.getProperty("voirresult"));
		voirResultat.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(liste.getSelectedValue()!=null)
				{
					ThreadCalcul tc = (ThreadCalcul)liste.getSelectedValue();
					if(tc.getStatut()==0)
						tc.AfficherResultat();
					else
						JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(),Application.langue.getProperty("calculnonfini"),null, JOptionPane.INFORMATION_MESSAGE);
						
				}
			}
		});
		
		JButton stop = new JButton(Application.langue.getProperty("stop"));
		
		stop.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				if(liste.getSelectedValue()!=null)
				{
					ThreadCalcul tc = (ThreadCalcul)liste.getSelectedValue();
					if(tc.getStatut()==0)
						{
							espaceDessin.threadCalculs.remove(tc);
							FenetrePrincipale.getInstance().retireThreadCalcul(tc);
							majThread();
						}
					else
						{
							tc.interrupt();
							espaceDessin.threadCalculs.remove(tc);
							FenetrePrincipale.getInstance().retireThreadCalcul(tc);
							majMethodeResolution();
							majThread();
						}
				}
			}
		});
		
		JPanel boutons = new JPanel();
		boutons.add(voirResultat);
		boutons.add(stop);
		
		pThread.setLayout(new GridLayout(2,1));
		
		pThread.add(sp);
		pThread.add(boutons);
		setVisible(false);
		setVisible(true);
		repaint();
	}
	
	
	/**
	 * Methode qui verifie que chaque etat a une etiquette differente
	 * @param s
	 * @return boolean
	 */
	public boolean contrainteEtiquetteEtatDifferent(String s)
	{
		ArrayList<EtatGraphique> liste_etat = espaceDessin.graphe.listeEtats;
		for(int i=0; i<liste_etat.size(); i++)
		{
			if(liste_etat.get(i).getEtiquette().compareTo(s)==0)
			{
				return false;
			}
		}
		return true;
	}
	
}
