/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.chemin;

import graphe.Graphe;
import ihm.Application;
import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.Transition;
import java.awt.Color;
import java.io.Serializable;
import java.util.*;
import javax.swing.JOptionPane;



import cout.CoutP1;
import calcul.Matrice;
/**
 * Classe servant a representer les chemins dans le graphe
 * @author giraudet-docet
 *
 */
public class Chemin implements Serializable
{
	private static final long serialVersionUID = 1L;
	private String nom;
	private ArrayList<Transition> chemin;
	private Graphe graphe;
	private Matrice<Intervalle> matriceIntervalle;
	private Graphe grapheGenere;
	private Intervalle _ideal;
	private CoutP1 cout;
	private int duree;
	private boolean activate;
	
	public Chemin(Graphe g, String s, int n)
	{
		super();
		nom = s;
		chemin = new ArrayList<Transition>();
		graphe = g;
		matriceIntervalle = null;
		duree = n;
		activate = true;
	}
	
	public ArrayList<Transition> getListeTransitions()
	{
		return chemin;
	}
	
	public void setColorTransition()
	{
		for(int i=0;i<chemin.size();i++)
			chemin.get(i).setColorSel(Color.red);
	}
	
	public void setActivation(boolean b)
	{
		activate = b;
	}
	public boolean getActivation()
	{
		return activate;
	}
	public void setDuree(int n)
	{
		duree=n;
	}
	
	
	public boolean add(Transition t)
	{
		if(chemin.size()==0)
		{
			chemin.add(t);
			return true;
		}
		else
		{
			Transition tlast = chemin.get(chemin.size()-1);
			if(tlast.getEtatArrivee().equals(t.getEtatDepart()))
			{
				chemin.add(t);
				return true;
			}
			else
			{
				JOptionPane.showMessageDialog(null, Application.langue.getProperty("erreurC1")+t.toString()+Application.langue.getProperty("erreurC2"), Application.langue.getProperty("erreur"), JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}
	}
	
	public void setIntervalle(double min, double max)
	{
		_ideal = new Intervalle(min,max);
	}
	public Intervalle getIntervalle()
	{
		return _ideal;
	}
	
	public boolean contains(EtatGraphique eg)
	{
		boolean trouve = false;
		int cpt=0;
		while(!trouve && cpt<chemin.size())
		{
			trouve = chemin.get(cpt).getEtatArrivee().equals(eg);
			cpt++;
		}
		return trouve;
	}
	
	public Transition get(int i)
	{
		return chemin.get(i);
	}
	
	public void remove(int n)
	{
		chemin.remove(n);
	}
	
	public int size()
	{
		return chemin.size();
	}
	
	public boolean finPossible()
	{
		if(chemin.size()>0)
			return !chemin.get(chemin.size()-1).estBoucle();
		else
			return true;
	}
	
	public void setNom(String s)
	{
		nom = s;
	}
	public String getNom()
	{
		return nom;
	}
	
	public String toString()
	{
		return nom;
	}
	
	
	/**
	 * fonction qui permet de generer le graphe modifier par le chemin
	 * @return
	 */
	public Graphe genereGraphe()
	{
		if(grapheGenere==null)
		{
			grapheGenere = new Graphe();
			
			//Recopie des tats du graphe initial
			for(int i=0; i<graphe.getNombreEtats(); i++)
			{
				EtatGraphique e = new EtatGraphique(graphe.listeEtats.get(i).getEtiquette());
				e.setNum(graphe.listeEtats.get(i).getnum());
				grapheGenere.ajoutEtat(e);
			}
			//Cration des nouveaux tats
			for(int i=0; i<chemin.size(); i++)
			{
				EtatGraphique e = new EtatGraphique(chemin.get(i).getEtatArrivee().getEtiquette() + "'");
				e.setNum(-chemin.get(i).getEtatArrivee().getnum());
				grapheGenere.ajoutEtat(e);
			}
			//Recopie des transitions, sauf la premiere du chemin
			for(int i=0; i<graphe.listeTransitions.size(); i++)
			{
				if(!graphe.listeTransitions.get(i).equals(chemin.get(0)))
				{
					Transition tbase = graphe.listeTransitions.get(i);
					Transition tcopy = new Transition(grapheGenere.rechercheEtatNum(tbase.getEtatDepart().getnum()),grapheGenere.rechercheEtatNum(tbase.getEtatArrivee().getnum()));
					tcopy.getIntervalles().add(tbase.getIntervalleProbabilite());
					grapheGenere.ajoutTransition(tcopy);
				}
			}
			/*
			 * Proprit C1
			 */
			//Cration de la premiere transition du chemin
			Transition tf = new Transition(grapheGenere.rechercheEtatNum(chemin.get(0).getEtatDepart().getnum()),grapheGenere.rechercheEtatNum(-chemin.get(0).getEtatArrivee().getnum()));
			tf.getIntervalles().add(graphe.rechercheTransition(chemin.get(0).getEtatDepart(), chemin.get(0).getEtatArrivee()).getIntervalleProbabilite());
			grapheGenere.ajoutTransition(tf);
			for(int i=1; i<chemin.size(); i++)
			{
				Transition t = new Transition(grapheGenere.rechercheEtatNum(-chemin.get(i).getEtatDepart().getnum()),grapheGenere.rechercheEtatNum(-chemin.get(i).getEtatArrivee().getnum()));
				t.getIntervalles().add(chemin.get(i).getIntervalleProbabilite());
				grapheGenere.ajoutTransition(t);
			}
			/*
			 * Proprit C2
			 */
			for(int i=1; i<chemin.size(); i++)
			{
				int nEtat = chemin.get(i).getEtatDepart().getnum();
				int nSuiv = chemin.get(i).getEtatArrivee().getnum();
				EtatGraphique eCours = grapheGenere.rechercheEtatNum(nEtat);
				for(int j=0; j<eCours.getTransitionsSortantes().size(); j++)
				{
					if(eCours.getTransitionsSortantes().get(j).getEtatArrivee().getnum()!=nSuiv)
					{
						Transition t = new Transition(grapheGenere.rechercheEtatNum(-nEtat),grapheGenere.rechercheEtatNum(eCours.getTransitionsSortantes().get(j).getEtatArrivee().getnum()));
						t.getIntervalles().add(eCours.getTransitionsSortantes().get(j).getIntervalleProbabilite());
						grapheGenere.ajoutTransition(t);
					}
				}
			}
			EtatGraphique e = chemin.get(chemin.size()-1).getEtatArrivee();
			for(int i=0; i<e.getTransitionsSortantes().size(); i++)
			{
				Transition t = new Transition(grapheGenere.rechercheEtatNum(-e.getnum()),grapheGenere.rechercheEtatNum(e.getTransitionsSortantes().get(i).getEtatArrivee().getnum()));
				t.getIntervalles().add(e.getTransitionsSortantes().get(i).getIntervalleProbabilite());
				grapheGenere.ajoutTransition(t);
			}
		}
		return grapheGenere;
	}
	

	public void completeGraphe(Graphe g)
	{
		for(int i=0; i<g.listeTransitions.size(); i++)
		{
			g.listeTransitions.get(i).getEtatDepart().getTransitionsSortantes().add(g.listeTransitions.get(i));
			g.listeTransitions.get(i).getEtatArrivee().getTransitionsEntrantes().add(g.listeTransitions.get(i));
		}
	}
	
	/**
	 * genere la nouvelle matrice intervalle pour le chemin
	 * @return
	 */
	public Matrice<Intervalle> genereMatrice()
	{
		if(matriceIntervalle==null)
			matriceIntervalle = genereGraphe().construireMatriceIntervalle();
		return matriceIntervalle;
	}
	
	/**
	 * fonction qui genere la matrice de cout correspondante au graphe genere par le chemin 
	 * @return
	 */
	public Matrice<Integer> genereMatriceCout()
	{
		
		int size = graphe.getNombreEtats()+chemin.size();
		Matrice<Integer> mRes = new Matrice<Integer>(size,size);
		Transition tfin=chemin.get(chemin.size()-1);
		int numetatfin = size-1;
		int numetatdeb;
		if(chemin.size()==1)
			numetatdeb = tfin.getEtatDepart().getnum()-1;
		else
			numetatdeb =size-2;
		for(int i=0; i<mRes.getNbLigne(); i++)
			{
			for(int j=0; j<mRes.getNbColonne(); j++)
			
				{
				if(i==numetatdeb && j==numetatfin)
					mRes.setCase(i, j, 1);
				else	
					mRes.setCase(i, j, 0);	
				}
			}
		return mRes;
	}
	
	public CoutP1 getCout()
	{
		if(cout==null)
			cout = new CoutP1(Application.langue.getProperty("couts")+nom,genereMatriceCout(),_ideal,duree);
		return cout;
	}
	
	/**
	 * fonction qui permet de transformer une matrice m correspondante au graphe
	 * en une matrice qui correspond au graphe genere par le chemin
	 * @param m
	 * @return
	 */
	public Matrice<Double> transformeMatrice(Matrice<Double> m)
	{
		int N = m.getNbLigne();
		Matrice<Double> mRes = new Matrice<Double>(N+chemin.size(),N+chemin.size());
		//recopie sans la premiere transition du chemin
		for(int i=0; i<N; i++)
		{
			for(int j=0; j<N; j++)
			{
				if(!(i+1==chemin.get(0).getEtatDepart().getnum() && j+1==chemin.get(0).getEtatArrivee().getnum()))
					mRes.setCase(i, j, m.getCase(i, j));
			}
		}
		//recopie de la premiere decalee
		mRes.setCase(chemin.get(0).getEtatDepart().getnum()-1, N, m.getCase(chemin.get(0).getEtatDepart().getnum()-1, chemin.get(0).getEtatArrivee().getnum()-1));
		//recopie des aretes sortantes
		for(int i=0; i<chemin.size(); i++)
		{
			int etat = chemin.get(i).getEtatArrivee().getnum();
			for(int l=0; l<N; l++)
			{
				mRes.setCase(N+i, l, m.getCase(etat-1,l));
			}
		}
		for(int i=1; i<chemin.size(); i++)
		{
			int etatD = chemin.get(i).getEtatDepart().getnum();
			int etatA = chemin.get(i).getEtatArrivee().getnum();
			mRes.setCase(N+i-1, N+i, m.getCase(etatD-1, etatA-1));
			mRes.setCase(N+i-1, etatA-1, null);
		}
		for(int i=0; i<mRes.getNbLigne(); i++)
		{
			for(int j=0; j<mRes.getNbColonne(); j++)
			{
				if(mRes.getCase(i, j)==null)
					mRes.setCase(i, j, 0d);
			}
		}
		return mRes;
	}
	
	
	
}
