/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.dessin;

import ihm.Application;
import ihm.FenetrePrincipale;
import ihm.chemin.Chemin;
import ihm.etat.EtatGraphique;
import ihm.transition.Transition;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
/**
 * Classe qui herite de la classe DessinGrapheState et qui correspond a l'etat
 * ou on supprime
 * @author Docet Giraudet
 *
 */
public class DessinGrapheStateSuppression extends DessinGrapheState{
	
	private ArrayList<Chemin> lc;
	private EtatGraphique etat;
	private Transition f;
	
	public DessinGrapheStateSuppression(DessinGraphe dg)
	{
		super(dg);
	}
	
public  void mouseClicked(MouseEvent e) {
}
	
	public  void mousePressed(MouseEvent e) {
	}
	/**
	 * On redefinie la methode ou on supprime l'objet sur lequel on a clique,
	 * on affiche un avertissement dans le cas ou la suppression d'un etat ou d'une transition
	 * supprime un chemin
	 */
	public  void mouseReleased(MouseEvent e) {
		etat = _dg.rechercheEtatGraphique(e.getX(),e.getY());
			if(etat!=null)
			{
				CheminsEtat(etat);
				if(lc.size()!=0)
				{
					DialogChemin(true);
				}
				else
				{
					_dg.removeTransitionEtat(etat);
					_dg.graphe.retireEtat(etat);
					_dg.repaint();
				}
			}
			else
			{
				f=_dg.rechercheTransitionGraphique(e.getX(),e.getY());
				
				if(f!=null)
				{
					CheminsTransition(f);
					if(lc.size()!=0)
					{
						DialogChemin(false);
					}
					else
					{
					_dg.graphe.supprimeTransition(f);
					f.getEtatArrivee().getTransitionsEntrantes().remove(f);
					f.getEtatDepart().getTransitionsSortantes().remove(f);
					_dg.getListeTransition().remove(f);
					}
					_dg.repaint();
				}
			}
			_dg.setState(new DessinGrapheStateDefault(_dg));
			
	}
	
	public  void mouseEntered(MouseEvent arg0) {
	}

	public  void mouseExited(MouseEvent e) {
	}
	
	public  void mouseDragged(MouseEvent e) {
	}
	
	public  void mouseMoved(MouseEvent arg0) {
	}
	
	private void CheminsEtat(EtatGraphique e)
	{
		lc = new ArrayList<Chemin>();
		ArrayList<Chemin> chemin = _dg.getChemins();
		for(int i=0;i<chemin.size();i++)
		{
			Chemin c = chemin.get(i);
			ArrayList<Transition> lt = c.getListeTransitions();
			for(int j=0;j<lt.size();j++)
			{
				Transition t = lt.get(j);
				if(t.getEtatArrivee().equals(e) || t.getEtatDepart().equals(e))
					if(!lc.contains(c))
					lc.add(c);
			}
				
		}
	}
	
	private void CheminsTransition(Transition t)
	{
		lc = new ArrayList<Chemin>();
		ArrayList<Chemin> chemin = _dg.getChemins();
		for(int i=0;i<chemin.size();i++)
		{
			Chemin c = chemin.get(i);
			ArrayList<Transition> lt = c.getListeTransitions();
				if(lt.contains(t))
					if(!lc.contains(c))
					lc.add(c);
		}
	}
	
	private void DialogChemin(final boolean et)
	{
		final JDialog jd = new JDialog(FenetrePrincipale.getInstance());
		JLabel jl;
		if(et)
			jl=new JLabel(Application.langue.getProperty("alerteDGSS1"));
		else
			jl=new JLabel(Application.langue.getProperty("alerteDGSS2"));
		JButton continuer = new JButton(Application.langue.getProperty("continuer"));
		JButton cancel = new JButton(Application.langue.getProperty("annuler"));
		continuer.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				for(int i=0;i<lc.size();i++)
				{
					_dg.getChemins().remove(lc.get(i));
				}
				_dg.menuTraitement.majChemins();
				if(et)
				{
					_dg.removeTransitionEtat(etat);
					_dg.graphe.retireEtat(etat);
					_dg.repaint();
				}
				else
				{
					_dg.graphe.supprimeTransition(f);
					f.getEtatArrivee().getTransitionsEntrantes().remove(f);
					f.getEtatDepart().getTransitionsSortantes().remove(f);
					_dg.getListeTransition().remove(f);
				}
				jd.dispose();
			}
		});
		cancel.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e)
			{
				jd.dispose();
			}
		});
		JPanel pBouton = new JPanel();
		pBouton.add(continuer);
		pBouton.add(cancel);
		JPanel pLabel = new JPanel();
		pLabel.add(jl);
		jd.add(pLabel,BorderLayout.CENTER);
		jd.add(pBouton,BorderLayout.SOUTH);
		jd.setLocationRelativeTo(FenetrePrincipale.getInstance());
		jd.setSize(500,100);
		jd.setVisible(true);
	}
}
