/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.etat;

import ihm.transition.Transition;
import javax.swing.*;
import java.awt.*;
import java.io.Serializable;
import java.util.ArrayList;
/**
 * Classe pour representer les etats, qui herite de la classe JPanel et implemente
 * la classe Serializable
 * @author Docet Giraudet
 *
 */
public class EtatGraphique extends JPanel implements Serializable 
{
	protected int _num;
	protected String etiquette;
	protected int pos_x;
	protected int pos_y;
	protected static int rayon=25;
	private ArrayList<Transition> transitionsEntrantes;
	private ArrayList<Transition> transitionsSortantes;
	//protected ArrayList<Intervalle> intervalles = new ArrayList<Intervalle>();
	public static int etatSuivant = 1;
	private static final long serialVersionUID = 1L;
	protected static Color c=Color.BLACK;
	protected Color csel;
	/**
	 * Constructeur de la classe qui prend en parametre une etiquette
	 * @param eti
	 */
	public EtatGraphique(String eti)
	{
		_num = etatSuivant++;
		pos_x=50;
		pos_y=50;
		setSize(60,60);
		transitionsEntrantes = new ArrayList<Transition>();
		transitionsSortantes = new ArrayList<Transition>();
		etiquette=eti;
	}
	
	public static void setRayon(int i)
	{
		rayon=i;
	}
	/**
	 * Methode pour recuperer l'etiquette de l'etat
	 * @return etiquette String
	 */
	public String getEtiquette()
	{
		return etiquette;
	}
	public void setEtiquette(String s)
	{
		etiquette = s;
	}
	
	public void setColorSelection(Color col)
	{
		csel=col;
	}
	
	public static void setColor(Color col)
	{
		c=col;
	}
	/**
	 * Methode pour modifier l'abscisse de l'etat
	 * @param n
	 */
	public void setX(int n)
	{
		pos_x=n;
	}
	/**
	 * Methode pour modifier l'ordonne de l'etat
	 * @param n
	 */
	public void setY(int n)
	{
		pos_y=n;
	}
	/**
	 * Methode pour modifier le numero de l'etat
	 * @param n
	 */
	public void setNum(int n)
	{
		_num=n;
	}
	/**
	 * Methode pour recuperer le numero de l'etat
	 * @return _num int
	 */
	public int getnum()
	{
		return _num;
	}
	/**
	 * Methode pour recuperer l'abscisse de l'etat
	 */
	public int getX()
	{
		return pos_x;
	}
	/**
	 * Methode pour recuperer l'ordonnee de l'etat
	 */
	public int getY()
	{
		return pos_y;
	}
	/**
	 * Methode pour recuperer le centre de l'etat
	 * @return Point
	 */
	public Point getCentre()
	{
		return new Point(pos_x+rayon,pos_y+rayon);
	}
	/**
	 * Methode pour recuperer le rayon de l'etat
	 * @return rayon int
	 */
	public int getRayon()
	{
		return rayon;
	}
	/**
	 * Methode pour recuperer la liste de transitions entrantes,
	 * c'est a dire les fleches arrivant sur l'etat
	 * @return transitionsEntrantes ArrayList<Transition>
	 */
	public ArrayList<Transition> getTransitionsEntrantes()
	{
		return transitionsEntrantes;
	}
	/**
	 * Methode pour recuperer la liste de transitions sortantes,
	 * c'est a dire les fleches partant de l'etat
	 * @return transitionsSortantes ArrayList<Transition>
	 */
	public ArrayList<Transition> getTransitionsSortantes()
	{
		return transitionsSortantes;
	}
	
	/**
	 * Methode pour ajouter des transitions entrantes
	 * @param f
	 */
	public void addTransitionsEntrantes(Transition f)
	{
		transitionsEntrantes.add(f);
	}
	/**
	 * Methode pour ajouter des transitions sortantes
	 * @param f
	 */
	public void addTransitionsSortantes(Transition f)
	{
		transitionsSortantes.add(f);
	}
	/**
	 * Methode pour retirer une transition entrante
	 * @param f
	 */
	public void removeTransitionsEntrantes(Transition f)
	{
		transitionsEntrantes.remove(f);
	}
	/**
	 * Methode pour retirer une transition sortante
	 * @param f
	 */
	public void removeTransitionsSortantes(Transition f)
	{
		transitionsSortantes.remove(f);
	}
	/**
	 * Methode pour dessiner un etat
	 * @param g
	 */
	public void dessineEtat( final Graphics g )
	{
		if(csel==null)
			g.setColor(c);
		else
			g.setColor(csel);
		g.drawOval(pos_x,pos_y,rayon*2,rayon*2);
		g.drawString(String.valueOf(_num),getCentre().x-(6*(""+_num).length())/2,getCentre().y+3);
		//g.drawString(etiquette,pos_x+25-(etiquette.length()/2),pos_y);
	}
	/**
	 * Methode pour determiner si on cliquer dans l'etat
	 * @param x
	 * @param y
	 * @return boolean
	 */
	public boolean estDansEtat (int x,int y)
	{
		if(new Point(x,y).distance(pos_x+rayon,pos_y+rayon)<=rayon)
		{
			return true;
		}
		else return false;
	}
	
	
	
	
	
	
	public String toString()
	{
		return "Etat n" + _num + " : " + etiquette;
	}
	
	public boolean equals(EtatGraphique e)
	{
		if(e==null)
			return false;
		else
			return (this.getEtiquette().compareTo(e.getEtiquette())==0);
	}
}
	

