/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.intervalle;

import java.io.Serializable;
import java.util.Random;

import javax.swing.JPanel;
/**
 * Classe servant a representer les intervalles
 * @author giraudet-docet
 *
 */
public class Intervalle implements Serializable,Comparable<Intervalle>
{
	private static final long serialVersionUID = 1L;
	private double min;
	private double max;
	private String description;

	public Intervalle()
	{
		min = 0;
		max = 100;
		description = "";
	}

	public Intervalle(int _min, int _max)
	{
		min=_min;
		max=_max;
	}

	public Intervalle (double _min,double _max)
	{
		min=_min;
		max=_max;
	}

	public Intervalle(String s)
	{
		min = 0;
		max = 100;
		description = s;
	}

	public Intervalle somme(Intervalle i)
	{
		return new Intervalle(this.getMinDouble()+i.getMinDouble(),this.getMaxDouble()+i.getMaxDouble());
	}

	public boolean IntervalleUneValeur()
	{
		return (new Double(min).floatValue()-new Double(max).floatValue())==0;
	}

/**
 * fonction qui retourne une valeur au hasard dans l'intervalle
 * @return
 */
	public float RandomValeur()
	{
		if(max-min==0)
			return (float)min;
		else
		{
			Random r = new Random();
			int x=r.nextInt((int)(max*10-min*10)+1);
			return (float)((float)(x/10)+min);
		}
	}

	public boolean IntervalleNul()
	{
		return (min==0)&&(max==0);
	}

	public void setMin(int n)
	{
		min = n;
	}
	public void setMax(int n)
	{
		max = n;
	}
	public void setDescription(String s)
	{
		description = s;
	}

	public int getMin()
	{
		return  (int) min;
	}
	public int getMax()
	{
		return (int) max;
	}

	public Double getMinDouble()
	{
		return min;
	}

	public Double getMaxDouble()
	{
		return max;
	}

	public Double getMinProba()
	{
		return min/100;
	}
	public Double getMaxProba()
	{
		return max/100;
	}

	public String getDescription()
	{
		return description;
	}

	public JPanel toPanel()
	{
		JPanel p = new JPanel();
		return p;
	}

	public String toString()
	{
		return "[" + min + "," + max +"]";
	}

	public String AfficheMatrice()
	{
		return "["+ min + ";" + max +"]";
	}


	public int compareTo(Intervalle i) {
		if((i.max==max)&&(i.min==min))
			return 0;
		else return 1;
	}

	public Double moyenne()
	{
		return new Double((max+min)/2);
	}

	public boolean estDansIntervalle(double d)
	{
		return ((d>=min) && (d<=max));
	}
}
