/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package ihm.intervalle;


import ihm.Application;

import java.awt.*;
import java.io.Serializable;
import javax.swing.*;
import javax.swing.event.*;
import util.JIntField;
/**
 * Classe servant a la representation graphique d'un intervalle
 * c est a dire un panel contenant les sliders pour bouger les bornes de l'intervalle
 * @author giraudet-e
 *
 */
public class IntervalleGraphique extends JPanel implements Serializable
{
	static final long serialVersionUID = 0;
	protected JLabel lbMinimum = new JLabel(Application.langue.getProperty("minig"));
	protected JLabel lbMaximum = new JLabel(Application.langue.getProperty("maxig"));
	protected JSlider jsMinimum = new JSlider();
	protected JSlider jsMaximum = new JSlider();
	protected JIntField ifValMin = new JIntField(3);
	protected JIntField ifValMax = new JIntField(3);
	protected Intervalle intervalle;
		
	public IntervalleGraphique(Intervalle i)
	{
		super();
		intervalle = i;
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createTitledBorder(Application.langue.getProperty("iig")+" " + intervalle.getDescription()));
		setLayout(new GridLayout(2,2));
		
		jsMinimum.setValue(i.getMin());
		jsMaximum.setValue(i.getMax());
		
		jsMinimum.setMaximum(i.getMax());
		jsMaximum.setMinimum(i.getMin());
		
		jsMinimum.setPaintTicks(true);
		jsMinimum.setMinorTickSpacing(5);
		jsMinimum.setMajorTickSpacing(20);
		jsMinimum.setPaintLabels(true);
		
		jsMaximum.setPaintTicks(true);
		jsMaximum.setMinorTickSpacing(5);
		jsMaximum.setMajorTickSpacing(20);
		jsMaximum.setPaintLabels(true);
		
		ifValMin.setInt(jsMinimum.getValue());
		ifValMax.setInt(jsMaximum.getValue());
		ifValMin.setEditable(false);
		ifValMax.setEditable(false);
		
		jsMinimum.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				jsMaximum.setMinimum(jsMinimum.getValue());
				ifValMin.setInt(jsMinimum.getValue());
			}
		});
		jsMaximum.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e){
				jsMinimum.setMaximum(jsMaximum.getValue());
				ifValMax.setInt(jsMaximum.getValue());
			}
		});
				
		//ifValMin.getDocument().addDocumentListener(new FieldSliderListener(jsMinimum,ifValMin));
		//ifValMax.getDocument().addDocumentListener(new FieldSliderListener(jsMaximum,ifValMax));
						
		JPanel pMin = new JPanel();
		pMin.add(lbMinimum);
		pMin.add(jsMinimum);
		pMin.add(ifValMin);
		JPanel pMax = new JPanel();			
		pMax.add(lbMaximum);
		pMax.add(jsMaximum);
		pMax.add(ifValMax);
		
		add(pMin);
		add(pMax);
	}
	
	public void sauverValeurs()
	{
		intervalle.setMin(jsMinimum.getValue());
		intervalle.setMax(jsMaximum.getValue());
	}
	
	public String getDescription()
	{
		return intervalle.getDescription();
	}
	public void setDescription(String s)
	{
		intervalle.setDescription(s);
	}
	public Intervalle getIntervalle()
	{
		return intervalle;
	}
		
}
