/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
 */

package thread;

import ihm.Application;
import ihm.FenetrePrincipale;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.WindowConstants;



import cout.CoutMult;
import graphe.Graphe;
import calcul.CourbeGnuPlot;
import calcul.Matrice;
import calcul.MethodeResolution;
import calcul.RechercheLocale;
import dialog.DialogAfficheGraphe;
/**
 * Classe servant a lancer un calcul pour afficher une courbe
 * @author giraudet
 *
 */
public class StartCalculCourbeGraph extends ThreadCalcul{

	private  ArrayList<CoutMult> lcm;
	private Matrice<Double> m;
	private static int num=0;
	private int numero;
	private double epsilon;
	private Graphe graphe;
	private CourbeGnuPlot cgp;
	private RechercheLocale rl;
	private boolean first;
	private boolean fin;
	private boolean affiche;

	public StartCalculCourbeGraph(Graphe g,ArrayList<CoutMult> _clm,double eps)
	{
		fin=false;
		num++;
		numero=num;
		statut=Application.langue.getProperty("statut");
		lcm = _clm;
		nom=Application.langue.getProperty("nomscc")+" "+numero;
		epsilon=eps;
		graphe=g;
		first=true;
		affiche=true;
	}


	public void run()
	{
		if(!isInterrupted()&&!fin)
		{
			ArrayList<MethodeResolution> lmr = new ArrayList<MethodeResolution>();
			lmr.add(FenetrePrincipale.getInstance().getEspaceTravail().getDessinGraphe().methodes.get(1));
			rl = new RechercheLocale(graphe.construireMatriceIntervalle(),lmr,epsilon,numero);
			rl.start();
			while(!rl.s.tryAcquire())
			{
				if(isInterrupted()&&!fin) 
					rl.interrupt(); 
				//else System.out.println("interrupt "+isInterrupted());
			}
			m=rl.resultat;
			if(m==null)
			{
				JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(),Application.langue.getProperty("resultsrl1")+"°"+numero+" "+Application.langue.getProperty("resultsrl2"),Application.langue.getProperty("resultsrl3"),JOptionPane.WARNING_MESSAGE);
				FenetrePrincipale.getInstance().retireThreadCalcul(this);
				this.interrupt();
			}
			else
			{
				statut=Application.langue.getProperty("statutfin");
				FenetrePrincipale.getInstance().getEspaceTravail().getMenuTraitement().majThread();
			}
		}
	
	}

	@SuppressWarnings("static-access")
	@Override
	public void AfficherResultat() {
		if(first)
		{
			first=false;
			final JDialog choixCourbe = new JDialog(FenetrePrincipale.getInstance());
			JPanel pBouton = new JPanel();
			JButton valider = new JButton(Application.langue.getProperty("valider"));
			JButton annuler = new JButton(Application.langue.getProperty("annuler"));
			pBouton.add(valider);
			pBouton.add(annuler);
			JPanel pTout = new JPanel();
			JLabel jl= new JLabel(Application.langue.getProperty("labelscc"));
			JPanel pL = new JPanel();
			pL.add(jl);
			pTout.add(pL);
			pTout.setLayout(new BoxLayout(pTout,BoxLayout.Y_AXIS));
			final ArrayList<Boolean> lb = new ArrayList<Boolean>();
			for(int i=0;i<lcm.size();i++)
			{
				final int t=i;
				final JCheckBox jcb = new JCheckBox(lcm.get(i).getNom());
				lb.add(jcb.isSelected());
				jcb.addActionListener(new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						lb.set(t, jcb.isSelected());
					}
				}
				);
				pTout.add(jcb);
			}
			annuler.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					choixCourbe.dispose();
				}
			});

			valider.addActionListener(new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					choixCourbe.dispose();
					ArrayList<CoutMult> lca = new ArrayList<CoutMult>();
					for(int i=0;i<lb.size();i++)
						if(lb.get(i))
							lca.add(lcm.get(i));
					try {
						cgp = new CourbeGnuPlot(lca,m,numero);
						cgp.run();
						if(!isInterrupted()&&!fin)
							while(!cgp.sfin.tryAcquire()){if(!isInterrupted()&&!fin)break;}
						final DialogAfficheGraphe dag = new DialogAfficheGraphe("graph"+numero+".png","graph"+numero+".ps");
						affiche=false;
						dag.addWindowListener(new WindowAdapter(){
							public void windowClosing(WindowEvent e){
								dag.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
								affiche=true;
								dag.dispose();
							}
						});
					} catch (IOException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}	
				}
			}
			);
			pTout.add(pBouton);
			JScrollPane jsp = new JScrollPane(pTout);
			choixCourbe.setTitle(this.nom);
			choixCourbe.add(jsp);
			choixCourbe.setLocationRelativeTo(FenetrePrincipale.getInstance());
			choixCourbe.setSize(300,200);
			choixCourbe.setVisible(true);
		}
		else
		{
			if(cgp!=null&&!first&&affiche)
			{
			final DialogAfficheGraphe dag = new DialogAfficheGraphe("graph"+numero+".png","graph"+numero+".ps");
			affiche=false;
			dag.addWindowListener(new WindowAdapter(){
				public void windowClosing(WindowEvent e){
					dag.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
					affiche=true;
					dag.dispose();
				}
			});
			}
		else
			JOptionPane.showMessageDialog(FenetrePrincipale.getInstance(),Application.langue.getProperty("erreurscc"));
		}
	}

	public void interrupt()
	{
		if(cgp!=null)
			cgp.interrupt();
		if(rl!=null)
			rl.interrupt();
		fin=true;
	}

}
