/*
Copyright Bourdon Jeremie 
contributors : Giraudet Etienne, Docet Eric

jeremie.bourdon@univ-nantes.fr
eric.docet@etu.univ-nantes.fr
etienne.giraudet@etu.univ-nants.fr

This software is a computer program whose purpose is to do computations
on genetic graphs.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can use, 
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.
*/

package util;

import ihm.chemin.Chemin;
import ihm.etat.EtatGraphique;
import ihm.intervalle.Intervalle;
import ihm.transition.RelationProba;
import ihm.transition.Transition;

import java.util.ArrayList;


import thread.ThreadCalcul;

import calcul.MethodeResolution;
import cout.Cout;
/**
 * Classe servant a convertir des arraylist en pour JList
 * @author giraudet-docet
 *
 */
public class ConvertArrayToList 
{
	/**
	 * Convertit une liste de chemins
	 * @param lc
	 * @return
	 */
	public static Chemin[] convertListChemins(ArrayList<Chemin> lc)
	{
		Chemin[] ls = new Chemin[lc.size()];
		for(int i = 0; i<lc.size(); i++)
			ls[i] = lc.get(i);
		return ls;
	}
	
	/**
	 * Convertit une liste de cout
	 * @param lc
	 * @return
	 */
	public static Cout[] convertListCout(ArrayList<Cout> lc)
	{
		Cout[] ls = new Cout[lc.size()];
		for(int i=0;i<lc.size();i++)
			ls[i]=lc.get(i);
		return ls;
	}	
	
	/**
	 * Convertit une liste de methode de resolution
	 * @param lm
	 * @return
	 */
	public static MethodeResolution[] convertListMethode(ArrayList<MethodeResolution> lm)
	{
		MethodeResolution[] res = new MethodeResolution[lm.size()];
		for(int i=0;i<lm.size();i++)
			res[i]=lm.get(i);
		return res;
	}
	
	/**
	 * Convertit une liste de transition
	 * @param lt
	 * @return
	 */
	public static Transition[] convertListTransitions(ArrayList<Transition> lt)
	{
		if(lt!=null)
		{
			Transition[] t = new Transition[lt.size()];
			for(int i = 0; i<lt.size(); i++)
				t[i] = lt.get(i);
			return t;
		}
		else
			return new Transition[0];
	}

	/**
	 * Convertit une liste d'etat
	 * @param le
	 * @return
	 */
	public static EtatGraphique[] convertListChemins(ArrayList<EtatGraphique> le)
	{
		EtatGraphique[] res = new EtatGraphique[le.size()];
		for(int i = 0; i<le.size(); i++)
			res[i] = le.get(i);
		return res;
	}
	
	/**
	 * Convertit une liste de relation
	 * @param lp
	 * @return
	 */
	public static RelationProba[] convertListRelations(ArrayList<RelationProba> lp)
	{
		if(lp!=null)
		{
			RelationProba[] r = new RelationProba[lp.size()];
			for(int i = 0; i<lp.size(); i++)
				r[i] = lp.get(i);
			return r;
		}
		else
			return new RelationProba[0];
	}
	
	/**
	 * Convertit une liste d'intervalle
	 * @param li
	 * @return
	 */
	public static Intervalle[] convertStringList(ArrayList<Intervalle> li)
	{
		Intervalle[] ls = new Intervalle[li.size()];
		for(int i = 0; i<li.size(); i++)
			ls[i] = li.get(i);
		return ls;
	}

	/**
	 * Convertit une liste de thread calculs
	 * @param threadCalculs
	 * @return
	 */
	public static ThreadCalcul[] convertListThreadCalcul(ArrayList<ThreadCalcul> threadCalculs) 
	{
		if(threadCalculs!=null)
		{
		ThreadCalcul[] ls = new ThreadCalcul[threadCalculs.size()];
		for(int i = 0; i<threadCalculs.size(); i++)
			ls[i] = threadCalculs.get(i);
		return ls;
		}
		else
			return new ThreadCalcul[0];
	}
	
}
